@@ -13,7 +13,7 @@
  *  * Neither the name of the ALICE Project-Team nor the names of its
  *  contributors may be used to endorse or promote products derived from this
  *  software without specific prior written permission.
- * 
+ *
  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -54,66 +54,66 @@ extern "C" {
 /******************************************************************************/
 /* Abstract matrix interface */
 
-struct NLMatrixStruct;
-typedef struct NLMatrixStruct* NLMatrix;
+    struct NLMatrixStruct;
+    typedef struct NLMatrixStruct* NLMatrix;
 
 /**
  * \brief Function pointer type for matrix destructors.
- */    
-typedef void(*NLDestroyMatrixFunc)(NLMatrix M);    
+ */
+    typedef void(*NLDestroyMatrixFunc)(NLMatrix M);
 
 /**
  * \brief Function pointer type for matrix x vector product.
  */
-typedef void(*NLMultMatrixVectorFunc)(NLMatrix M, const double* x, double* y);
+    typedef void(*NLMultMatrixVectorFunc)(NLMatrix M, const double* x, double* y);
 
 #define NL_MATRIX_SPARSE_DYNAMIC 0x1001
 #define NL_MATRIX_CRS            0x1002
-#define NL_MATRIX_SUPERLU_EXT    0x1003    
-#define NL_MATRIX_CHOLMOD_EXT    0x1004    
+#define NL_MATRIX_SUPERLU_EXT    0x1003
+#define NL_MATRIX_CHOLMOD_EXT    0x1004
 #define NL_MATRIX_FUNCTION       0x1005
 #define NL_MATRIX_OTHER          0x1006
-    
+
 /**
  * \brief The base class for abstract matrices.
  */
-struct NLMatrixStruct {
-    /**
-     * \brief number of rows 
-     */    
-    NLuint m;
+    struct NLMatrixStruct {
+        /**
+         * \brief number of rows
+         */
+        NLuint m;
 
-    /**
-     * \brief number of columns 
-     */    
-    NLuint n;
+        /**
+         * \brief number of columns
+         */
+        NLuint n;
 
-    /**
-     * \brief Matrix type
-     * \details One of NL_MATRIX_SPARSE_DYNAMIC, 
-     *  NL_MATRIX_CRS, NL_MATRIX_SUPERLU_EXT,
-     *  NL_CHOLDMOD_MATRIX_EXT, NL_MATRIX_FUNCTION,
-     *  NL_MATRIX_OTHER
-     */
-    NLenum type;
+        /**
+         * \brief Matrix type
+         * \details One of NL_MATRIX_SPARSE_DYNAMIC,
+         *  NL_MATRIX_CRS, NL_MATRIX_SUPERLU_EXT,
+         *  NL_CHOLDMOD_MATRIX_EXT, NL_MATRIX_FUNCTION,
+         *  NL_MATRIX_OTHER
+         */
+        NLenum type;
 
-    /**
-     * \brief Destructor
-     */
-    NLDestroyMatrixFunc destroy_func;
+        /**
+         * \brief Destructor
+         */
+        NLDestroyMatrixFunc destroy_func;
 
-    /**
-     * \brief Matrix x vector product
-     */
-    NLMultMatrixVectorFunc mult_func;
-};
+        /**
+         * \brief Matrix x vector product
+         */
+        NLMultMatrixVectorFunc mult_func;
+    };
 
 /**
  * \brief Deletes a matrix
  * \details If \p M is NULL, then the function does nothing
  * \param[in] M the matrix to be deleted
  */
-NLAPI void NLAPIENTRY nlDeleteMatrix(NLMatrix M);
+    NLAPI void NLAPIENTRY nlDeleteMatrix(NLMatrix M);
 
 /**
  * \brief Computes a matrix x vector product
@@ -121,10 +121,10 @@ NLAPI void NLAPIENTRY nlDeleteMatrix(NLMatrix M);
  * \param[in] x the vector to be multiplied by the matrix
  * \param[out] y the result
  */
-NLAPI void NLAPIENTRY nlMultMatrixVector(
-    NLMatrix M, const double* x, double* y
-);
-    
+    NLAPI void NLAPIENTRY nlMultMatrixVector(
+        NLMatrix M, const double* x, double* y
+    );
+
 /******************************************************************************/
 /* Dynamic arrays for sparse row/columns */
 
@@ -132,76 +132,76 @@ NLAPI void NLAPIENTRY nlMultMatrixVector(
  * \brief Represents a coefficient in a sparse matrix
  * \relates NLSparseMatrix
  */
-typedef struct  {
-    /**
-     * \brief index of the coefficient.
-     */    
-    NLuint index;
+    typedef struct  {
+        /**
+         * \brief index of the coefficient.
+         */
+        NLuint index;
 
-    /**
-     * \brief value of the coefficient. 
-     */    
-    NLdouble value; 
-} NLCoeff;
+        /**
+         * \brief value of the coefficient.
+         */
+        NLdouble value;
+    } NLCoeff;
 
 /**
  * \brief Represents a row or a column of a sparse matrix
  * \relates NLSparseMatrix
  */
-typedef struct {
-    /**
-     * \brief number of coefficients. 
-     */    
-    NLuint size;
-    
-    /** 
-     * \brief number of coefficients that can be 
-     * stored without reallocating memory.
-     */    
-    NLuint capacity;
+    typedef struct {
+        /**
+         * \brief number of coefficients.
+         */
+        NLuint size;
 
-    /**
-     * \brief the array of coefficients, with enough
-     * space to store capacity coefficients.
-     */
-    NLCoeff* coeff;  
-} NLRowColumn;
+        /**
+         * \brief number of coefficients that can be
+         * stored without reallocating memory.
+         */
+        NLuint capacity;
+
+        /**
+         * \brief the array of coefficients, with enough
+         * space to store capacity coefficients.
+         */
+        NLCoeff* coeff;
+    } NLRowColumn;
 
 /**
  * \brief Constructs a new NLRowColumn
- * \param[in,out] c a pointer to an 
+ * \param[in,out] c a pointer to an
  *  uninitialized NLRowColumn
  * \relates NLRowColumn
  */
-NLAPI void NLAPIENTRY nlRowColumnConstruct(NLRowColumn* c);
+    NLAPI void NLAPIENTRY nlRowColumnConstruct(NLRowColumn* c);
 
 /**
  * \brief Destroys a NLRowColumn
- * \details Only the memory allocated by the 
+ * \details Only the memory allocated by the
  *  NLRowColumn is freed. The NLRowColumn structure
  *  is not freed.
  * \param[in,out] c a pointer to an NLRowColumn
  * \relates NLRowColumn
  */
-NLAPI void NLAPIENTRY nlRowColumnDestroy(NLRowColumn* c);
+    NLAPI void NLAPIENTRY nlRowColumnDestroy(NLRowColumn* c);
 
 /**
  * \brief Allocates additional storage for
  *  the coefficients of an NLRowColumn
- * \details Operates like the class vector of the C++ standard library, 
+ * \details Operates like the class vector of the C++ standard library,
  *  by doubling the capacity each time it is needed. This amortizes
  *  the cost of the growing operations as compared to re-allocating
  *  at each element insertion
  * \param[in,out] c a pointer to an NLRowColumn
  * \relates NLRowColumn
  */
-NLAPI void NLAPIENTRY nlRowColumnGrow(NLRowColumn* c);
+    NLAPI void NLAPIENTRY nlRowColumnGrow(NLRowColumn* c);
 
 /**
- * \brief Adds a coefficient to an NLRowColumn.    
+ * \brief Adds a coefficient to an NLRowColumn.
  * \details Performs the following operation:
  *  \f$ a_i \leftarrow a_i + value \f$. If the NLRowColumn
- *  already has a coefficient with index \p index, then 
+ *  already has a coefficient with index \p index, then
  *  the value is added to that coefficient, else a new
  *  coefficient is created. Additional storage is allocated
  *  as need be.
@@ -210,9 +210,9 @@ NLAPI void NLAPIENTRY nlRowColumnGrow(NLRowColumn* c);
  * \param[in] value value of the coefficient
  * \relates NLRowColumn
  */
-NLAPI void NLAPIENTRY nlRowColumnAdd(
-    NLRowColumn* c, NLuint index, NLdouble value
-);
+    NLAPI void NLAPIENTRY nlRowColumnAdd(
+        NLRowColumn* c, NLuint index, NLdouble value
+    );
 
 /**
  * \brief Appends a coefficient to an NLRowColumn    .
@@ -224,9 +224,9 @@ NLAPI void NLAPIENTRY nlRowColumnAdd(
  * \param[in] value value of the coefficient
  * \relates NLRowColumn
  */
-NLAPI void NLAPIENTRY nlRowColumnAppend(
-    NLRowColumn* c, NLuint index, NLdouble value
-);
+    NLAPI void NLAPIENTRY nlRowColumnAppend(
+        NLRowColumn* c, NLuint index, NLdouble value
+    );
 
 /**
  * \brief Zeroes an NLRowColumn.
@@ -235,7 +235,7 @@ NLAPI void NLAPIENTRY nlRowColumnAppend(
  * \param[in,out] c a pointer to an NLRowColumn
  * \relates NLRowColumn
  */
-NLAPI void NLAPIENTRY nlRowColumnZero(NLRowColumn* c);
+    NLAPI void NLAPIENTRY nlRowColumnZero(NLRowColumn* c);
 
 /**
  * \brief Zeroes an NLRowColumn and deallocates the memory
@@ -244,7 +244,7 @@ NLAPI void NLAPIENTRY nlRowColumnZero(NLRowColumn* c);
  * \param[in,out] c a pointer to an NLRowColumn
  * \relates NLRowColumn
  */
-NLAPI void NLAPIENTRY nlRowColumnClear(NLRowColumn* c);
+    NLAPI void NLAPIENTRY nlRowColumnClear(NLRowColumn* c);
 
 /**
  * \brief Sorts the coefficients of an NLRowColumn
@@ -252,7 +252,7 @@ NLAPI void NLAPIENTRY nlRowColumnClear(NLRowColumn* c);
  * \param[in,out] c a pointer to an NLRowColumn
  * \relates NLRowColumn
  */
-NLAPI void NLAPIENTRY nlRowColumnSort(NLRowColumn* c);
+    NLAPI void NLAPIENTRY nlRowColumnSort(NLRowColumn* c);
 
 /******************************************************************************/
 /* Compressed Row Storage */
@@ -267,75 +267,75 @@ NLAPI void NLAPIENTRY nlRowColumnSort(NLRowColumn* c);
 #else
     typedef NLuint NLuint_big;
 #endif
-    
-    
+
+
 /**
- * \brief A compact self-contained storage for 
+ * \brief A compact self-contained storage for
  *  sparse matrices.
  * \details Unlike with NLSparseMatrix, it is not possible
  *  to add new coefficients in an NLCRSMatrix.
  */
-typedef struct {
-    /**
-     * \brief number of rows 
-     */    
-    NLuint m;
-    
-    /**
-     * \brief number of columns
-     */    
-    NLuint n;
+    typedef struct {
+        /**
+         * \brief number of rows
+         */
+        NLuint m;
 
-    /**
-     * \brief Matrix type
-     * \details One of NL_MATRIX_DYNAMIC, NL_MATRIX_CRS, NL_MATRIX_SUPERLU_EXT,
-     *  NL_CHOLDMOD_MATRIX_EXT
-     */
-    NLenum type;
-    
-    /**
-     * \brief destructor
-     */
-    NLDestroyMatrixFunc destroy_func;
+        /**
+         * \brief number of columns
+         */
+        NLuint n;
 
-    /**
-     * \brief Matrix x vector product
-     */
-    NLMultMatrixVectorFunc mult_func;
-    
-    /**
-     * \brief array of coefficient values, 
-     * size = NNZ (number of non-zero coefficients)
-     */    
-    NLdouble* val;    
+        /**
+         * \brief Matrix type
+         * \details One of NL_MATRIX_DYNAMIC, NL_MATRIX_CRS,
+	 *  NL_MATRIX_SUPERLU_EXT, NL_CHOLDMOD_MATRIX_EXT
+         */
+        NLenum type;
 
-    /**
-     * \brief row pointers, size = m+1 
-     */    
-    NLuint_big* rowptr;
+        /**
+         * \brief destructor
+         */
+        NLDestroyMatrixFunc destroy_func;
 
-    /**
-     * \brief column indices, size = NNZ 
-     */    
-    NLuint* colind;
+        /**
+         * \brief Matrix x vector product
+         */
+        NLMultMatrixVectorFunc mult_func;
 
-    /**
-     * \brief number of slices, used by parallel spMv
-     */    
-    NLuint nslices;
+        /**
+         * \brief array of coefficient values,
+         * size = NNZ (number of non-zero coefficients)
+         */
+        NLdouble* val;
 
-    /** 
-     * \brief slice pointers, size = nslices + 1, 
-     * used by parallel spMv 
-     */    
-    NLuint* sliceptr;
+        /**
+         * \brief row pointers, size = m+1
+         */
+        NLuint_big* rowptr;
 
-    /**
-     * \brief NL_TRUE if symmetric storage is used,
-     *  NL_FALSE otherwise.
-     */
-    NLboolean symmetric_storage;
-} NLCRSMatrix;
+        /**
+         * \brief column indices, size = NNZ
+         */
+        NLuint* colind;
+
+        /**
+         * \brief number of slices, used by parallel spMv
+         */
+        NLuint nslices;
+
+        /**
+         * \brief slice pointers, size = nslices + 1,
+         * used by parallel spMv
+         */
+        NLuint* sliceptr;
+
+        /**
+         * \brief NL_TRUE if symmetric storage is used,
+         *  NL_FALSE otherwise.
+         */
+        NLboolean symmetric_storage;
+    } NLCRSMatrix;
 
 /**
  * \brief Constructs a new NLCRSMatrix
@@ -347,9 +347,9 @@ typedef struct {
  *  (typically, nslices = number of cores)
  * \relates NLCRSMatrix
  */
-NLAPI void NLAPIENTRY nlCRSMatrixConstruct(
-    NLCRSMatrix* M, NLuint m, NLuint n, NLuint_big nnz, NLuint nslices
-);
+    NLAPI void NLAPIENTRY nlCRSMatrixConstruct(
+        NLCRSMatrix* M, NLuint m, NLuint n, NLuint_big nnz, NLuint nslices
+    );
 
 /**
  * \brief Constructs a new NLCRSMatrix with symmetric storage
@@ -358,9 +358,9 @@ NLAPI void NLAPIENTRY nlCRSMatrixConstruct(
  * \param[in] nnz number of non-zero coefficients
  * \relates NLCRSMatrix
  */
-NLAPI void NLAPIENTRY nlCRSMatrixConstructSymmetric(
-    NLCRSMatrix* M, NLuint n, NLuint_big nnz
-);
+    NLAPI void NLAPIENTRY nlCRSMatrixConstructSymmetric(
+        NLCRSMatrix* M, NLuint n, NLuint_big nnz
+    );
 
 
 /**
@@ -372,9 +372,9 @@ NLAPI void NLAPIENTRY nlCRSMatrixConstructSymmetric(
  * \param[in] n number of columns
  * \relates NLCRSMatrix
  */
-NLAPI void NLAPIENTRY nlCRSMatrixConstructPattern(
-    NLCRSMatrix* M, NLuint m, NLuint n
-);
+    NLAPI void NLAPIENTRY nlCRSMatrixConstructPattern(
+        NLCRSMatrix* M, NLuint m, NLuint n
+    );
 
 /**
  * \brief Constructs a new NLCRSMatrix with symmetric storage
@@ -385,18 +385,18 @@ NLAPI void NLAPIENTRY nlCRSMatrixConstructPattern(
  * \param[in] nnz number of non-zero coefficients
  * \relates NLCRSMatrix
  */
-NLAPI void NLAPIENTRY nlCRSMatrixConstructPatternSymmetric(
-    NLCRSMatrix* M, NLuint n
-);
+    NLAPI void NLAPIENTRY nlCRSMatrixConstructPatternSymmetric(
+        NLCRSMatrix* M, NLuint n
+    );
 
 /**
- * \brief Specifies the number of non-zero entries in the row of a 
+ * \brief Specifies the number of non-zero entries in the row of a
  *  matrix that was constructed by nlCRSMatrixConstructPattern() or
  *  nlCRSMatrixConstructPatternSymmetric().
  */
-NLAPI void NLAPIENTRY nlCRSMatrixPatternSetRowLength(
-    NLCRSMatrix* M, NLuint i, NLuint n
-);
+    NLAPI void NLAPIENTRY nlCRSMatrixPatternSetRowLength(
+        NLCRSMatrix* M, NLuint i, NLuint n
+    );
 
 
 /**
@@ -404,7 +404,7 @@ NLAPI void NLAPIENTRY nlCRSMatrixPatternSetRowLength(
  *  lengths).
  * \param[in] M a pointer to the NLCRSMatrix to be compiled.
  */
-NLAPI void NLAPIENTRY nlCRSMatrixPatternCompile(NLCRSMatrix* M);    
+    NLAPI void NLAPIENTRY nlCRSMatrixPatternCompile(NLCRSMatrix* M);
 
 /**
  * \brief Adds a coefficient to an NLSparseMatrix
@@ -414,36 +414,69 @@ NLAPI void NLAPIENTRY nlCRSMatrixPatternCompile(NLCRSMatrix* M);
  * \param[in] i index of the row
  * \param[in] j index of the column
  * \param[in] value the coefficient to be added
+ * \return the index in the CRS arrays where the value was inserted
  * \relates NLSparseMatrix
  */
-NLAPI void NLAPIENTRY nlCRSMatrixAdd(
-    NLCRSMatrix* M, NLuint i, NLuint j, NLdouble value
-);
+    NLAPI NLulong NLAPIENTRY nlCRSMatrixAdd(
+        NLCRSMatrix* M, NLuint i, NLuint j, NLdouble value
+    );
+
+/**
+ * \brief Adds a coefficient to an NLSparseMatrix at a known location
+ * \details Performs the following operation:
+ *  \$ a_{i,j} \leftarrow a_{i,j} + \mbox{value} \$
+ * \param[in,out] M a pointer to an NLSparseMatrix
+ * \param[in] i index of the row
+ * \param[in] j index of the column
+ * \param[in] value the coefficient to be added
+ * \param[in] index the index in the CRS arrays where the coefficient is stored
+ * \pre The coefficient at index \p index already exists and
+ *   has column index \p j
+ * \relates NLSparseMatrix
+ */
+    NLAPI void NLAPIENTRY nlCRSMatrixAddAt(
+        NLCRSMatrix* M, NLuint i, NLuint j, NLdouble value, NLulong index
+    );
+
+/**
+ * \brief Sets a coefficient to an NLSparseMatrix at a known row offset
+ * \details Performs the following operation:
+ *  \$ a_{i,j} \leftarrow a_{i,j} + \mbox{value} \$
+ * \param[in,out] M a pointer to an NLSparseMatrix
+ * \param[in] i index of the row
+ * \param[in] j index of the column
+ * \param[in] value the coefficient to be added
+ * \param[in] row_offset the offset of the CRS arrays starting from rowptr[i]
+ * \pre The coefficient in row \p i at \p row_offset was not previously set
+ * \relates NLSparseMatrix
+ */
+    NLAPI void NLAPIENTRY nlCRSMatrixSetCoefficientAtRowOffset(
+        NLCRSMatrix* M, NLuint i, NLuint j, NLdouble value, NLuint row_offset
+    );
 
-    
 /**
  * \brief Loads a NLCRSMatrix from a file
- * \param[out] M a pointer to an uninitialized NLCRSMatriix 
+ * \param[out] M a pointer to an uninitialized NLCRSMatriix
  * \param[in] filename the name of the file
  * \retval NL_TRUE on success
  * \retval NL_FALSE on error
  * \relates NLCRSMatrix
  */
-NLAPI NLboolean NLAPIENTRY nlCRSMatrixLoad(
-    NLCRSMatrix* M, const char* filename
-);
+    NLAPI NLboolean NLAPIENTRY nlCRSMatrixLoad(
+        NLCRSMatrix* M, const char* filename
+    );
 
 /**
  * \brief Saves a NLCRSMatrix into a file
- * \param[in] M a pointer to the NLCRSMatriix 
+ * \param[in] M a pointer to the NLCRSMatriix
  * \param[in] filename the name of the file
  * \retval NL_TRUE on success
  * \retval NL_FALSE on error
  * \relates NLCRSMatrix
  */
-NLAPI NLboolean NLAPIENTRY nlCRSMatrixSave(
-    NLCRSMatrix* M, const char* filename
-);
+    NLAPI NLboolean NLAPIENTRY nlCRSMatrixSave(
+        NLCRSMatrix* M, const char* filename
+    );
 
 /**
  * \brief Gets the number of non-zero coefficient
@@ -452,8 +485,8 @@ NLAPI NLboolean NLAPIENTRY nlCRSMatrixSave(
  * \return the number of non-zero coefficients in \p M
  * \relates NLCRSMatrix
  */
-NLAPI NLuint_big NLAPIENTRY nlCRSMatrixNNZ(NLCRSMatrix* M);
-    
+    NLAPI NLuint_big NLAPIENTRY nlCRSMatrixNNZ(NLCRSMatrix* M);
+
 /******************************************************************************/
 /* SparseMatrix data structure */
 
@@ -475,81 +508,81 @@ NLAPI NLuint_big NLAPIENTRY nlCRSMatrixNNZ(NLCRSMatrix* M);
  * \relates NLSparseMatrix
  */
 #define NL_MATRIX_STORE_SYMMETRIC     4
-    
-typedef struct {
-    /**
-     * \brief number of rows 
-     */    
-    NLuint m;
-    
-    /**
-     * \brief number of columns 
-     */    
-    NLuint n;
 
-    /**
-     * \brief Matrix type
-     * \details One of NL_MATRIX_DYNAMIC, NL_MATRIX_CRS, NL_MATRIX_SUPERLU_EXT,
-     *  NL_CHOLDMOD_MATRIX_EXT
-     */
-    NLenum type;
-    
-    /**
-     * \brief destructor
-     */
-    NLDestroyMatrixFunc destroy_func;
+    typedef struct {
+        /**
+         * \brief number of rows
+         */
+        NLuint m;
 
-    /**
-     * \brief Matrix x vector product
-     */
-    NLMultMatrixVectorFunc mult_func;
+        /**
+         * \brief number of columns
+         */
+        NLuint n;
 
-    
-    /**
-     * \brief number of elements in the diagonal 
-     */    
-    NLuint diag_size;
+        /**
+         * \brief Matrix type
+         * \details One of NL_MATRIX_DYNAMIC, NL_MATRIX_CRS, NL_MATRIX_SUPERLU_EXT,
+         *  NL_CHOLDMOD_MATRIX_EXT
+         */
+        NLenum type;
 
-    /**
-     * \brief Number of elements allocated to store the diagonal
-     */
-    NLuint diag_capacity;
-    
-    /**
-     * \brief indicates what is stored in this matrix 
-     */    
-    NLenum storage;
+        /**
+         * \brief destructor
+         */
+        NLDestroyMatrixFunc destroy_func;
 
-    /**
-     * \brief the rows if (storage & NL_MATRIX_STORE_ROWS), size = m,
-     * NULL otherwise
-     */     
-    NLRowColumn* row;
+        /**
+         * \brief Matrix x vector product
+         */
+        NLMultMatrixVectorFunc mult_func;
 
-    /** 
-     * \brief the columns if (storage & NL_MATRIX_STORE_COLUMNS), size = n,
-     * NULL otherwise
-     */     
-    NLRowColumn* column;
 
-    /**
-     * \brief the diagonal elements, size = diag_size 
-     */     
-    NLdouble*    diag;
+        /**
+         * \brief number of elements in the diagonal
+         */
+        NLuint diag_size;
 
-    /**
-     * \brief Number of row descriptors allocated in the row
-     *  array.
-     */
-    NLuint row_capacity;
+        /**
+         * \brief Number of elements allocated to store the diagonal
+         */
+        NLuint diag_capacity;
 
-    /**
-     * \brief Number of column descriptors allocated in the 
-     *  column array.
-     */
-    NLuint column_capacity;
-    
-} NLSparseMatrix;
+        /**
+         * \brief indicates what is stored in this matrix
+         */
+        NLenum storage;
+
+        /**
+         * \brief the rows if (storage & NL_MATRIX_STORE_ROWS), size = m,
+         * NULL otherwise
+         */
+        NLRowColumn* row;
+
+        /**
+         * \brief the columns if (storage & NL_MATRIX_STORE_COLUMNS), size = n,
+         * NULL otherwise
+         */
+        NLRowColumn* column;
+
+        /**
+         * \brief the diagonal elements, size = diag_size
+         */
+        NLdouble*    diag;
+
+        /**
+         * \brief Number of row descriptors allocated in the row
+         *  array.
+         */
+        NLuint row_capacity;
+
+        /**
+         * \brief Number of column descriptors allocated in the
+         *  column array.
+         */
+        NLuint column_capacity;
+
+    } NLSparseMatrix;
 
 
 /**
@@ -562,9 +595,9 @@ typedef struct {
  *   It can be later deallocated by nlDeleteMatrix().
  * \relates NLSparseMatrix
  */
-NLAPI NLMatrix NLAPIENTRY nlSparseMatrixNew(
-    NLuint m, NLuint n, NLenum storage
-);
+    NLAPI NLMatrix NLAPIENTRY nlSparseMatrixNew(
+        NLuint m, NLuint n, NLenum storage
+    );
 
 /**
  * \brief Constructs a new NLSparseMatrix
@@ -575,9 +608,9 @@ NLAPI NLMatrix NLAPIENTRY nlSparseMatrixNew(
  *  indicate what needs to be stored in the matrix.
  * \relates NLSparseMatrix
  */
-NLAPI void NLAPIENTRY nlSparseMatrixConstruct(
-    NLSparseMatrix* M, NLuint m, NLuint n, NLenum storage
-);
+    NLAPI void NLAPIENTRY nlSparseMatrixConstruct(
+        NLSparseMatrix* M, NLuint m, NLuint n, NLenum storage
+    );
 
 /**
  * \brief Destroys an NLSparseMatrix
@@ -586,7 +619,7 @@ NLAPI void NLAPIENTRY nlSparseMatrixConstruct(
  * \param[in,out] M a pointer to an NLSparseMatrix
  * \relates NLSparseMatrix
  */
-NLAPI void NLAPIENTRY nlSparseMatrixDestroy(NLSparseMatrix* M);
+    NLAPI void NLAPIENTRY nlSparseMatrixDestroy(NLSparseMatrix* M);
 
 /**
  * \brief Computes a matrix-vector product
@@ -595,10 +628,10 @@ NLAPI void NLAPIENTRY nlSparseMatrixDestroy(NLSparseMatrix* M);
  * \param[in] y where to store the result, size = A->m
  * \relates NLSparseMatrix
  */
-NLAPI void NLAPIENTRY nlSparseMatrixMult(
-    NLSparseMatrix* A, const NLdouble* x, NLdouble* y
-);    
-    
+    NLAPI void NLAPIENTRY nlSparseMatrixMult(
+        NLSparseMatrix* A, const NLdouble* x, NLdouble* y
+    );
+
 /**
  * \brief Adds a coefficient to an NLSparseMatrix
  * \details Performs the following operation:
@@ -609,9 +642,9 @@ NLAPI void NLAPIENTRY nlSparseMatrixMult(
  * \param[in] value the coefficient to be added
  * \relates NLSparseMatrix
  */
-NLAPI void NLAPIENTRY nlSparseMatrixAdd(
-    NLSparseMatrix* M, NLuint i, NLuint j, NLdouble value
-);
+    NLAPI void NLAPIENTRY nlSparseMatrixAdd(
+        NLSparseMatrix* M, NLuint i, NLuint j, NLdouble value
+    );
 
 /**
  * \brief Adds a matrix to another sparse matrix.
@@ -623,17 +656,17 @@ NLAPI void NLAPIENTRY nlSparseMatrixAdd(
  * \param[in] N a matrix. Needs to be either a NLSparseMatrix or
  *   a NLCRSMatrix.
  */
-NLAPI void NLAPIENTRY nlSparseMatrixAddMatrix(
-    NLSparseMatrix* M, double mul, const NLMatrix N
-);	
-    
+    NLAPI void NLAPIENTRY nlSparseMatrixAddMatrix(
+        NLSparseMatrix* M, double mul, const NLMatrix N
+    );
+
 /**
  * \brief Zeroes an NLSparseMatrix
  * \details The memory is not freed.
  * \param[in,out] M a pointer to the NLSparseMatrix to zero
  * \relates NLSparseMatrix
  */
-NLAPI void NLAPIENTRY nlSparseMatrixZero( NLSparseMatrix* M);
+    NLAPI void NLAPIENTRY nlSparseMatrixZero( NLSparseMatrix* M);
 
 /**
  * \brief Clears an NLSparseMatrix
@@ -641,7 +674,7 @@ NLAPI void NLAPIENTRY nlSparseMatrixZero( NLSparseMatrix* M);
  * \param[in,out] M a pointer to the NLSparseMatrix to zero
  * \relates NLSparseMatrix
  */
-NLAPI void NLAPIENTRY nlSparseMatrixClear( NLSparseMatrix* M);
+    NLAPI void NLAPIENTRY nlSparseMatrixClear( NLSparseMatrix* M);
 
 /**
  * \brief Gets the number of non-zero coefficient
@@ -650,26 +683,26 @@ NLAPI void NLAPIENTRY nlSparseMatrixClear( NLSparseMatrix* M);
  * \return the number of non-zero coefficients in \p M
  * \relates NLSparseMatrix
  */
-NLAPI NLuint_big NLAPIENTRY nlSparseMatrixNNZ( NLSparseMatrix* M);
+    NLAPI NLuint_big NLAPIENTRY nlSparseMatrixNNZ( NLSparseMatrix* M);
 
 /**
  * \brief Sorts the coefficients in an NLSParseMatrix
- * \param[in,out] M a pointer to the NLSparseMatrix 
+ * \param[in,out] M a pointer to the NLSparseMatrix
  * \relates NLSparseMatrix
  */
-NLAPI void NLAPIENTRY nlSparseMatrixSort( NLSparseMatrix* M);
+    NLAPI void NLAPIENTRY nlSparseMatrixSort( NLSparseMatrix* M);
 
 /**
  * \brief Adds a new empty row to a sparse matrix.
  * \param[in,out] M a pointer to the sparse matrix.
  */
-NLAPI void NLAPIENTRY nlSparseMatrixAddRow( NLSparseMatrix* M);
+    NLAPI void NLAPIENTRY nlSparseMatrixAddRow( NLSparseMatrix* M);
 
 /**
  * \brief Adds a new empty column to a sparse matrix.
  * \param[in,out] M a pointer to the sparse matrix.
  */
-NLAPI void NLAPIENTRY nlSparseMatrixAddColumn( NLSparseMatrix* M);
+    NLAPI void NLAPIENTRY nlSparseMatrixAddColumn( NLSparseMatrix* M);
 
 /**
  * \brief Adds a row of a sparse matrix to another row.
@@ -679,9 +712,9 @@ NLAPI void NLAPIENTRY nlSparseMatrixAddColumn( NLSparseMatrix* M);
  * \param[in] s scaling factor.
  * \param[in] i2 index of the other row.
  */
-NLAPI void NLAPIENTRY nlSparseMatrixMAddRow(
-    NLSparseMatrix* M, NLuint i1, double s, NLuint i2
-);
+    NLAPI void NLAPIENTRY nlSparseMatrixMAddRow(
+        NLSparseMatrix* M, NLuint i1, double s, NLuint i2
+    );
 
 /**
  * \brief Scales a row of a sparse matrix.
@@ -691,9 +724,9 @@ NLAPI void NLAPIENTRY nlSparseMatrixMAddRow(
  * \param[in] s scaling factor.
  * \pre M has row storage and has not column storage.
  */
-NLAPI void NLAPIENTRY nlSparseMatrixScaleRow(
-    NLSparseMatrix* M, NLuint i, double s
-);
+    NLAPI void NLAPIENTRY nlSparseMatrixScaleRow(
+        NLSparseMatrix* M, NLuint i, double s
+    );
 
 /**
  * \brief Zeroes a row of a sparse matrix.
@@ -702,37 +735,37 @@ NLAPI void NLAPIENTRY nlSparseMatrixScaleRow(
  * \param[in] i index of the row.
  * \pre M has row storage and has not column storage.
  */
-NLAPI void NLAPIENTRY nlSparseMatrixZeroRow(
-    NLSparseMatrix* M, NLuint i
-);
+    NLAPI void NLAPIENTRY nlSparseMatrixZeroRow(
+        NLSparseMatrix* M, NLuint i
+    );
+
 
-    
 /******************************************************************************/
 
 /**
  * \brief Creates a compressed row storage matrix from a dynamic sparse matrix.
- * \details The matrix \p M should have stored rows. If \p M has 
+ * \details The matrix \p M should have stored rows. If \p M has
  *  symmetric storage, then the constructed matrix also has symmetric storage.
  * \param[in] M the dynamic sparse matrix.
  * \return a pointer to the created NLCRSMatrix
  * \relates NLCRSMatrix
  */
-NLAPI NLMatrix NLAPIENTRY nlCRSMatrixNewFromSparseMatrix(NLSparseMatrix* M);    
+    NLAPI NLMatrix NLAPIENTRY nlCRSMatrixNewFromSparseMatrix(NLSparseMatrix* M);
 
 /**
  * \brief Creates a compressed row storage matrix from a dynamic sparse matrix.
- * \details The matrix \p M should have stored rows. It is supposed 
- *  to be symmetric. If it does not have symmetric storage, then its upper 
+ * \details The matrix \p M should have stored rows. It is supposed
+ *  to be symmetric. If it does not have symmetric storage, then its upper
  *  triangular part is ignored.
  * \param[in] M the dynamic sparse matrix.
  * \return a pointer to the created NLCRSMatrix
  * \relates NLCRSMatrix
  */
-NLAPI NLMatrix NLAPIENTRY nlCRSMatrixNewFromSparseMatrixSymmetric(
-    NLSparseMatrix* M
-);    
+    NLAPI NLMatrix NLAPIENTRY nlCRSMatrixNewFromSparseMatrixSymmetric(
+        NLSparseMatrix* M
+    );
+
 
-    
 /**
  * \brief Compresses a dynamic sparse matrix into a CRS matrix.
  * \details If the input matrix is not a dynamic sparse matrix, it is left
@@ -740,7 +773,7 @@ NLAPI NLMatrix NLAPIENTRY nlCRSMatrixNewFromSparseMatrixSymmetric(
  * \param[in,out] M a pointer to the matrix to be compressed
  * \relates NLMatrix
  */
-NLAPI void NLAPIENTRY nlMatrixCompress(NLMatrix* M);
+    NLAPI void NLAPIENTRY nlMatrixCompress(NLMatrix* M);
 
 /**
  * \brief Gets the number of non-zero entries in a matrix.
@@ -748,7 +781,7 @@ NLAPI void NLAPIENTRY nlMatrixCompress(NLMatrix* M);
  *  else it returns m*n
  * \return the number of non-zero entries in \p M
  */
-NLAPI NLuint_big NLAPIENTRY nlMatrixNNZ(NLMatrix M);
+    NLAPI NLuint_big NLAPIENTRY nlMatrixNNZ(NLMatrix M);
 
 /**
  * \brief Factorizes a matrix.
@@ -764,8 +797,8 @@ NLAPI NLuint_big NLAPIENTRY nlMatrixNNZ(NLMatrix M);
  *  nlMultMatrixVector() with the result, it solves a linear system (the result
  *  may be thought of as the inverse of \p M).
  */
-NLAPI NLMatrix NLAPIENTRY nlMatrixFactorize(NLMatrix M, NLenum solver);
-    
+    NLAPI NLMatrix NLAPIENTRY nlMatrixFactorize(NLMatrix M, NLenum solver);
+
 /******************************************************************************/
 
 /**
@@ -775,14 +808,14 @@ NLAPI NLMatrix NLAPIENTRY nlMatrixFactorize(NLMatrix M, NLenum solver);
 
 /**
  * \brief Creates a matrix implemented by a matrix-vector function.
- * \param[in] m number of rows 
+ * \param[in] m number of rows
  * \param[in] n number of columns
  * \param[in] func a function that implements the matrix x vector product,
  *  and that takes the right hand side and the left hand side as arguments.
- */	     
-NLAPI NLMatrix NLAPIENTRY nlMatrixNewFromFunction(
-    NLuint m, NLuint n, NLMatrixFunc func
-);	     
+ */
+    NLAPI NLMatrix NLAPIENTRY nlMatrixNewFromFunction(
+        NLuint m, NLuint n, NLMatrixFunc func
+    );
 
 /**
  * \brief Gets the function pointer that implements matrix x vector product.
@@ -790,33 +823,33 @@ NLAPI NLMatrix NLAPIENTRY nlMatrixNewFromFunction(
  * \return the pointer to the matrix x vector product function if M was created
  *   by nlMatrixNewFromFunction(), NULL otherwise
  */
-NLAPI NLMatrixFunc NLAPIENTRY nlMatrixGetFunction(NLMatrix M);
+    NLAPI NLMatrixFunc NLAPIENTRY nlMatrixGetFunction(NLMatrix M);
 
 /******************************************************************************/
 
 /**
  * \brief Creates a matrix that represents the product of two matrices.
- * \param[in] M an m times k matrix. 
- * \param[in] product_owns_M if set to NL_TRUE, then caller is no longer 
- *  responsible for the memory associated to M, and M will be destroyed 
+ * \param[in] M an m times k matrix.
+ * \param[in] product_owns_M if set to NL_TRUE, then caller is no longer
+ *  responsible for the memory associated to M, and M will be destroyed
  *  when the product will be destroyed. If it is set to NL_FALSE, then the
  *  product only keeps a reference to M, and the caller remains responsible
  *  for deallocating M.
- * \param[in] N a k times n matrix. 
- * \param[in] product_owns_N if set to NL_TRUE, then caller is no longer 
- *  responsible for the memory associated to N, and N will be destroyed 
+ * \param[in] N a k times n matrix.
+ * \param[in] product_owns_N if set to NL_TRUE, then caller is no longer
+ *  responsible for the memory associated to N, and N will be destroyed
  *  when the product will be destroyed. If it is set to NL_FALSE, then the
  *  product only keeps a reference to N, and the caller remains responsible
  *  for deallocating N.
  * \return an NLMatrix that represents the product between M and N
  */
-NLAPI NLMatrix NLAPIENTRY nlMatrixNewFromProduct(
-    NLMatrix M, NLboolean product_owns_M,
-    NLMatrix N, NLboolean product_owns_N
-);
+    NLAPI NLMatrix NLAPIENTRY nlMatrixNewFromProduct(
+        NLMatrix M, NLboolean product_owns_M,
+        NLMatrix N, NLboolean product_owns_N
+    );
 
 /******************************************************************************/
-    
+
 #ifdef __cplusplus
 }
 #endif
