@@ -13,7 +13,7 @@
  *  * Neither the name of the ALICE Project-Team nor the names of its
  *  contributors may be used to endorse or promote products derived from this
  *  software without specific prior written permission.
- * 
+ *
  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -45,6 +45,7 @@
 #include <geogram/basic/determinant.h>
 #include <geogram/basic/memory.h>
 #include <geogram/basic/assert.h>
+#include <initializer_list>
 
 #include <iostream>
 #include <cfloat>
@@ -69,7 +70,7 @@ namespace GEO {
     class vecng {
     public:
         /** \brief The dimension of the vector */
-        static const index_t dim = DIM;
+        static constexpr index_t dim = DIM;
 
         /** \brief This vector type */
         typedef vecng<DIM, T> vector_type;
@@ -132,6 +133,19 @@ namespace GEO {
             }
         }
 
+        /**
+         * \brief Constructs a vector from an initializer list.
+         * \param[in] Vi the initializer list, should contain DIM elements.
+         */
+        vecng(const std::initializer_list<T>& Vi) {
+            index_t i = 0;
+            for(auto& it: Vi) {
+                geo_debug_assert(i < DIM);
+                data()[i] = it;
+                ++i;
+            }
+        }
+
         /**
          * \brief Gets the vector dimension
          * \return the value of \p DIM
@@ -506,7 +520,7 @@ namespace GEO {
     class vecng<2, T> {
     public:
         /** \copydoc vecng::dim */
-        static const index_t dim = 2;
+        static constexpr index_t dim = 2;
 
         /** \copydoc vecng::vector_type */
         typedef vecng<dim, T> vector_type;
@@ -543,6 +557,16 @@ namespace GEO {
             y(v[1]) {
         }
 
+        /** \copydoc vecng::vecng(const std::initializer_list<T>) */
+        vecng(const std::initializer_list<T>& Vi) {
+            index_t i = 0;
+            for(auto& it: Vi) {
+                geo_debug_assert(i < dim);
+                data()[i] = it;
+                ++i;
+            }
+        }
+
         /** \copydoc vecng::length2() const */
         inline T length2() const {
             return x * x + y * y;
@@ -649,6 +673,12 @@ namespace GEO {
             return data()[i];
         }
 
+        /** \brief Optimizes coordinate representation */
+        void optimize() {
+            Numeric::optimize_number_representation(x);
+            Numeric::optimize_number_representation(y);
+        }
+
         /** \brief Vector x coordinate */
         T x;
         /** \brief Vector y coordinate */
@@ -701,7 +731,7 @@ namespace GEO {
     class vecng<3, T> {
     public:
         /** \copydoc vecng::dim */
-        static const index_t dim = 3;
+        static constexpr index_t dim = 3;
 
         /** \copydoc vecng::vector_type */
         typedef vecng<dim, T> vector_type;
@@ -742,6 +772,16 @@ namespace GEO {
             z(v[2]) {
         }
 
+        /** \copydoc vecng::vecng(const std::initializer_list<T>) */
+        vecng(const std::initializer_list<T>& Vi) {
+            index_t i = 0;
+            for(auto& it: Vi) {
+                geo_debug_assert(i < dim);
+                data()[i] = it;
+                ++i;
+            }
+        }
+
         /** \copydoc vecng::length2() const */
         inline T length2() const {
             return x * x + y * y + z * z;
@@ -853,6 +893,13 @@ namespace GEO {
             return data()[i];
         }
 
+        /** \brief Optimizes coordinate representation */
+        void optimize() {
+            Numeric::optimize_number_representation(x);
+            Numeric::optimize_number_representation(y);
+            Numeric::optimize_number_representation(z);
+        }
+
         /** \brief Vector x coordinate */
         T x;
         /** \brief Vector y coordinate */
@@ -911,7 +958,7 @@ namespace GEO {
     class vecng<4, T> {
     public:
         /** \copydoc vecng::dim */
-        static const index_t dim = 4;
+        static constexpr index_t dim = 4;
 
         /** \copydoc vecng::vector_type */
         typedef vecng<dim, T> vector_type;
@@ -956,6 +1003,16 @@ namespace GEO {
             w(v[3]) {
         }
 
+        /** \copydoc vecng::vecng(const std::initializer_list<T>) */
+        vecng(const std::initializer_list<T>& Vi) {
+            index_t i = 0;
+            for(auto& it: Vi) {
+                geo_debug_assert(i < dim);
+                data()[i] = it;
+                ++i;
+            }
+        }
+
         /** \copydoc vecng::length2() const */
         inline T length2() const {
             return x * x + y * y + z * z + w * w;
@@ -1128,7 +1185,8 @@ namespace GEO {
     /**
      * \brief Reads a vector from a stream
      * \details This reads \p DIM coordinates from the input stream \p in and
-     * stores them in vector \p v
+     * stores them in vector \p v. Understands both "x y z" and
+     *  "[x, y, z]" formats.
      * \param[in] in the input stream
      * \param[out] v the vector to read
      * \return a reference to the input stream \p in
@@ -1138,12 +1196,56 @@ namespace GEO {
     inline std::istream& operator>> (
         std::istream& in, GEO::vecng<DIM, T>& v
     ) {
+        char c;
+        while(isspace(in.peek())) {
+            in.get(c);
+        }
+        if(in.peek() == '[') {
+            in.get(c);
+        }
+        while(isspace(in.peek())) {
+            in.get(c);
+        }
         for(index_t i = 0; i < DIM; i++) {
             in >> v[i];
+            while(isspace(in.peek())) {
+                in.get(c);
+            }
+            if(in.peek() == ',') {
+                in.get(c);
+            }
+            while(isspace(in.peek())) {
+                in.get(c);
+            }
+        }
+        if(in.peek() == ']') {
+            in.get(c);
         }
         return in;
     }
+
+    /************************************************************************/
+
+    namespace Numeric {
+
+        template<class T>
+        inline void optimize_number_representation(
+            vecng<2,T>& v
+        ) {
+            v.optimize();
+        }
+
+        template<class T>
+        inline void optimize_number_representation(
+            vecng<3,T>& v
+        ) {
+            v.optimize();
+        }
+
+    }
+
+    /************************************************************************/
+
 }
 
 #endif
-
