@@ -13,7 +13,7 @@
  *  * Neither the name of the ALICE Project-Team nor the names of its
  *  contributors may be used to endorse or promote products derived from this
  *  software without specific prior written permission.
- * 
+ *
  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -56,16 +56,16 @@ namespace GEO {
 
     /**
      * \brief Implementation of Delaunay in 2d.
-     * \details This package uses concepts inspired by 
+     * \details This package uses concepts inspired by
      *  two triangulation softwares, CGAL and tetgen,
      *  described in the following references. This package follows the
      *  idea used in CGAL of traversing the cavity from inside, since
      *  it traverses less triangles than when traversing from outside.
-     *  - Jean-Daniel Boissonnat, Olivier Devillers, Monique Teillaud, 
-     *   and Mariette Yvinec. Triangulations in CGAL. 
+     *  - Jean-Daniel Boissonnat, Olivier Devillers, Monique Teillaud,
+     *   and Mariette Yvinec. Triangulations in CGAL.
      *   In Proc. 16th Annu. ACM Sympos. Comput. Geom., pages 11-18, 2000.
-     *  - Hang Si, Constrained Delaunay trianglesl mesh generation and 
-     *   refinement. Finite elements in Analysis and Design, 
+     *  - Hang Si, Constrained Delaunay trianglesl mesh generation and
+     *   refinement. Finite elements in Analysis and Design,
      *   46 (1-2):33--46, 2010.
      *
      *  Note that the algorithm here does not support vertex deletion nor
@@ -74,33 +74,33 @@ namespace GEO {
      *
      *  The core algorithm used in this code, CGAL and tetgen was
      *  independently and simultaneously discovered by Bowyer and Watson:
-     *  - Adrian Bowyer, "Computing Dirichlet tessellations", 
-     *   Comput. J., vol. 24, no 2, 1981, p. 162-166 
-     *  - David F. Watson, "Computing the n-dimensional Delaunay tessellation 
-     *   with application to Voronoi polytopes", Comput. J., vol. 24, 
+     *  - Adrian Bowyer, "Computing Dirichlet tessellations",
+     *   Comput. J., vol. 24, no 2, 1981, p. 162-166
+     *  - David F. Watson, "Computing the n-dimensional Delaunay tessellation
+     *   with application to Voronoi polytopes", Comput. J., vol. 24,
      *   no 2, 1981, p. 167-172
      *
-     *  The spatial reordering method, that dramatically increases the 
+     *  The spatial reordering method, that dramatically increases the
      *  performances, also used in this code, CGAL and tetgen was introduced
      *  in the following references. The second one is a smart implementation
      *  based on the std::nth_element() function of the STL, that inspired
      *  the compute_BRIO_ordering() function of this package.
      *  - Nina Amenta, Sunghee Choi and Gunter Rote, "Incremental constructions
      *   con brio", ACM Symposium on Computational Geometry 2003.
-     *  - Christophe Delage and Olivier Devillers. Spatial Sorting. 
-     *   In CGAL User and Reference Manual. CGAL Editorial Board, 
+     *  - Christophe Delage and Olivier Devillers. Spatial Sorting.
+     *   In CGAL User and Reference Manual. CGAL Editorial Board,
      *   3.9 edition, 2011
      *
-     *  The locate() function is based on the following two references. 
+     *  The locate() function is based on the following two references.
      *  The first one randomizes the choice of the next triangle.
-     *  The second one uses an inexact locate() function to initialize 
+     *  The second one uses an inexact locate() function to initialize
      *  the exact one (it is called "structural filtering"). The first
      *  idea is used in both CGAL and tetgen, and the second one is used
      *  in CGAL.
      *  - Walking in a triangulation, O Devillers, S Pion, M Teillaud
      *    17th Annual Symposium on Computational geometry, 106-114
      *  - Stefan Funke , Kurt Mehlhorn and Stefan Naher, "Structural filtering,
-     *    a paradigm for efficient and exact geometric programs", 
+     *    a paradigm for efficient and exact geometric programs",
      *    Comput. Geom., 1999
      */
     class GEOGRAM_API Delaunay2d : public Delaunay {
@@ -120,42 +120,42 @@ namespace GEO {
          */
         Delaunay2d(coord_index_t dimension = 2);
 
-	/**
-	 * \copydoc Delaunay::set_vertices()
-	 */
+        /**
+         * \copydoc Delaunay::set_vertices()
+         */
         void set_vertices(
             index_t nb_vertices, const double* vertices
         ) override;
 
-	/**
-	 * \copydoc Delaunay::nearest_vertex()
-	 */
+        /**
+         * \copydoc Delaunay::nearest_vertex()
+         */
         index_t nearest_vertex(const double* p) const override;
 
-	/**
-	 * \brief Tests whether the Laguerre diagram has empty cells.
-	 * \details If the Laguerre diagram has empty cells and
-	 *  abort_if_empty_cell is set, then computation is stopped, 
-	 *  and all the queries on the Laguerre diagram will not work 
-	 *  (including the non-empty cells).
-	 * \retval true if the Laguerre diagram has empty cells.
-	 * \retval false otherwise.
-	 */
-	bool has_empty_cells() const {
-	    return has_empty_cells_;
-	}
+        /**
+         * \brief Tests whether the Laguerre diagram has empty cells.
+         * \details If the Laguerre diagram has empty cells and
+         *  abort_if_empty_cell is set, then computation is stopped,
+         *  and all the queries on the Laguerre diagram will not work
+         *  (including the non-empty cells).
+         * \retval true if the Laguerre diagram has empty cells.
+         * \retval false otherwise.
+         */
+        bool has_empty_cells() const {
+            return has_empty_cells_;
+        }
 
 
-	/**
-	 * \brief Specifies behavior if an empty cell is detected.
-	 * \param[in] x if set, then computation is aborted as soon
-	 *  as an empty cell is detected.
-	 * \details only happens in RegularTriangulation/Laguerre diagram.
-	 */
-	void abort_if_empty_cell(bool x) {
-	    abort_if_empty_cell_ = x;
-	}
-	
+        /**
+         * \brief Specifies behavior if an empty cell is detected.
+         * \param[in] x if set, then computation is aborted as soon
+         *  as an empty cell is detected.
+         * \details only happens in RegularTriangulation/Laguerre diagram.
+         */
+        void abort_if_empty_cell(bool x) {
+            abort_if_empty_cell_ = x;
+        }
+
     protected:
 
         /**
@@ -164,7 +164,7 @@ namespace GEO {
          *  specifying a hint. This constant indicates that
          *  no hint is given.
          */
-        static const index_t NO_TRIANGLE = index_t(-1);
+        static constexpr index_t NO_TRIANGLE = index_t(-1);
 
         /**
          * \brief Finds in the pointset a set of three non-colinear
@@ -200,14 +200,14 @@ namespace GEO {
          *  of index -1) or NO_TRIANGLE if the virtual triangles
          *  were previously removed.
          */
-         index_t locate(
+        index_t locate(
             const double* p, index_t hint = NO_TRIANGLE,
             bool thread_safe = false,
             Sign* orient = nullptr
-         ) const;
-         
+        ) const;
+
         /**
-         * \brief Finds the triangle that (approximately) 
+         * \brief Finds the triangle that (approximately)
          *  contains a point using inexact predicates.
          * \details The result of this function can be used as a hint
          *  for locate(). It accelerates locate as compared to calling
@@ -215,7 +215,7 @@ namespace GEO {
          *  filtering".
          * \param[in] p a pointer to the coordinates of the point
          * \param[in] max_iter maximum number of traversed tets
-         * \return the index of a triangle that (approximately) 
+         * \return the index of a triangle that (approximately)
          *  contains \p p.
          *  If the point is outside the convex hull of
          *  the inserted so-far points, then the returned triangle
@@ -223,9 +223,9 @@ namespace GEO {
          *  of index -1) or NO_TRIANGLE if the virtual triangles
          *  were previously removed.
          */
-         index_t locate_inexact(
-             const double* p, index_t hint, index_t max_iter
-         ) const;
+        index_t locate_inexact(
+            const double* p, index_t hint, index_t max_iter
+        ) const;
 
         /**
          * \brief Inserts a point in the triangulation.
@@ -235,7 +235,7 @@ namespace GEO {
          * \return the index of one of the triangles incident to
          *  point \p v
          */
-         index_t insert(index_t v, index_t hint = NO_TRIANGLE);
+        index_t insert(index_t v, index_t hint = NO_TRIANGLE);
 
         /**
          * \brief Determines the list of triangles in conflict
@@ -252,65 +252,65 @@ namespace GEO {
          *  adjacent to the boundary of the conflict zone
          * \param[out] first the index of the first triangle in conflict
          * \param[out] last the index of the last triangle in conflict
-         *  The other triangles are linked, and can be traversed 
-         *  from \p first by using triangle_next() until \p last or END_OF_LIST 
+         *  The other triangles are linked, and can be traversed
+         *  from \p first by using triangle_next() until \p last or END_OF_LIST
          *  is reached.
          *  The conflict zone can be empty under two circumstances:
          *  - the vertex \p v already exists in the triangulation
          *  - the triangulation is weighted and \p v is not visible
          *  in either cases, both \p first and \p last contain END_OF_LIST
          */
-         void find_conflict_zone(
-             index_t v, 
-             index_t t, const Sign* orient,
-             index_t& t_bndry, index_t& e_bndry,
-             index_t& first, index_t& last
-         );
-         
-         /**
-          * \brief This function is used to implement find_conflict_zone.
-          * \details This function detects the neighbors of \p t that are
-          *  in the conflict zone and calls itself recursively on them.
-          * \param[in] p the point to be inserted
-          * \param[in] t index of a triangle in the fonflict zone
-          * \param[out] t_bndry a triangle adjacent to the
-          *  boundary of the conflict zone
-          * \param[out] e_bndry the edge along which t_bndry is
-          *  adjacent to the boundary of the conflict zone
-          * \param[out] first the index of the first triangle in conflict
-          * \param[out] last the index of the last triangle in conflict
-          * \pre The triangle \p t was already marked as 
-          *  conflict (triangle_is_in_list(t))
-          */
-         void find_conflict_zone_iterative(
-             const double* p, index_t t,
-             index_t& t_bndry, index_t& e_bndry,
-             index_t& first, index_t& last
-         );
+        void find_conflict_zone(
+            index_t v,
+            index_t t, const Sign* orient,
+            index_t& t_bndry, index_t& e_bndry,
+            index_t& first, index_t& last
+        );
 
-         
-         /**
-          * \brief Creates a star of triangles filling the conflict
-          *  zone.
-          * \details For each triangle edge on the border of the
-          *  conflict zone, a new triangle is created, resting on
-          *  the edge and incident to vertex \p v. The function is 
-          *  called recursively until the entire conflict zone is filled.
-          * \param[in] v the index of the point to be inserted
-          * \param[in] t_bndry index of a triangle on the border
-          *  of the conflict zone.
-          * \param[in] e_bndry index of the facet along which \p t_bndry
-          *  is incident to the border of the conflict zone
-          * \return the index of one the newly created triangles
-          */
-         index_t stellate_conflict_zone(
-             index_t v, 
-             index_t t_bndry, index_t e_bndry
-         );
-         
-         // _________ Combinatorics - new and delete _________________________
+        /**
+         * \brief This function is used to implement find_conflict_zone.
+         * \details This function detects the neighbors of \p t that are
+         *  in the conflict zone and calls itself recursively on them.
+         * \param[in] p the point to be inserted
+         * \param[in] t index of a triangle in the fonflict zone
+         * \param[out] t_bndry a triangle adjacent to the
+         *  boundary of the conflict zone
+         * \param[out] e_bndry the edge along which t_bndry is
+         *  adjacent to the boundary of the conflict zone
+         * \param[out] first the index of the first triangle in conflict
+         * \param[out] last the index of the last triangle in conflict
+         * \pre The triangle \p t was already marked as
+         *  conflict (triangle_is_in_list(t))
+         */
+        void find_conflict_zone_iterative(
+            const double* p, index_t t,
+            index_t& t_bndry, index_t& e_bndry,
+            index_t& first, index_t& last
+        );
 
-         /**
+
+        /**
+         * \brief Creates a star of triangles filling the conflict
+         *  zone.
+         * \details For each triangle edge on the border of the
+         *  conflict zone, a new triangle is created, resting on
+         *  the edge and incident to vertex \p v. The function is
+         *  called recursively until the entire conflict zone is filled.
+         * \param[in] v the index of the point to be inserted
+         * \param[in] t_bndry index of a triangle on the border
+         *  of the conflict zone.
+         * \param[in] e_bndry index of the facet along which \p t_bndry
+         *  is incident to the border of the conflict zone
+         * \return the index of one the newly created triangles
+         */
+        index_t stellate_conflict_zone(
+            index_t v,
+            index_t t_bndry, index_t e_bndry
+        );
+
+        // _________ Combinatorics - new and delete _________________________
+
+        /**
          * \brief Maximum valid index for a triangle.
          * \details This includes not only real triangles,
          *  but also the virtual ones on the border, the conflict
@@ -331,13 +331,13 @@ namespace GEO {
          *  to be not in any list.
          * A triangle can be:
          *  - in a list (cell_next_[t] & NOT_IN_LIST_BIT == 0)
-         *  - not in a list and not marked 
-         *    (cell_next_[t] & NOT_IN_LIST_BIT != 0) && 
+         *  - not in a list and not marked
+         *    (cell_next_[t] & NOT_IN_LIST_BIT != 0) &&
          *    (cell_next_[t] != cur_stamp_)
          *  - not in a list and marked
          *    (cell_next_[t] == cur_stamp_)
          */
-        static const index_t NOT_IN_LIST  = index_t(~0);
+        static constexpr index_t NOT_IN_LIST  = index_t(~0);
 
         /**
          * \brief If cell_next_[t] & NOT_IN_LIST_BIT != 0,
@@ -348,20 +348,20 @@ namespace GEO {
          *  that were detected as non-conflict when inserting a point.
          * A triangle can be:
          *  - in a list (cell_next_[t] & NOT_IN_LIST_BIT == 0)
-         *  - not in a list and not marked 
-         *    (cell_next_[t] & NOT_IN_LIST_BIT != 0) && 
+         *  - not in a list and not marked
+         *    (cell_next_[t] & NOT_IN_LIST_BIT != 0) &&
          *    (cell_next_[t] != cur_stamp_)
          *  - not in a list and marked
          *    (cell_next_[t] == cur_stamp_)
          */
-        static const index_t NOT_IN_LIST_BIT = index_t(1u << 31);
+        static constexpr index_t NOT_IN_LIST_BIT = index_t(1u << 31);
 
         /**
          * \brief Symbolic value of the cell_next_ field
          *  that indicates the end of list in a linked
          *  list of triangles.
          */
-        static const index_t END_OF_LIST = ~(NOT_IN_LIST_BIT);
+        static constexpr index_t END_OF_LIST = ~(NOT_IN_LIST_BIT);
 
 
         /**
@@ -430,7 +430,7 @@ namespace GEO {
          *  belongs to.
          * \details Triangles can be linked, it is used to manage
          *  both the free list that recycles deleted triangles and
-         *  the list of triangles in conflict with the inserted 
+         *  the list of triangles in conflict with the inserted
          *  point.
          * \param[in] t the index of the triangle
          */
@@ -446,7 +446,7 @@ namespace GEO {
          * \details The three other vertices then correspond to a
          *  facet on the convex hull of the points.
          */
-        static const signed_index_t VERTEX_AT_INFINITY = -1;
+        static constexpr signed_index_t VERTEX_AT_INFINITY = -1;
 
         /**
          * \brief Tests whether a given triangle
@@ -459,12 +459,12 @@ namespace GEO {
          * \retval false otherwise
          */
         bool triangle_is_finite(index_t t) const {
-            return 
+            return
                 cell_to_v_store_[3 * t]     >= 0 &&
                 cell_to_v_store_[3 * t + 1] >= 0 &&
                 cell_to_v_store_[3 * t + 2] >= 0 ;
         }
-        
+
         /**
          * \brief Tests whether a triangle is
          *  a real one.
@@ -491,11 +491,11 @@ namespace GEO {
          */
         bool triangle_is_virtual(index_t t) const {
             return
-            !triangle_is_free(t) && (
-		cell_to_v_store_[3 * t] == VERTEX_AT_INFINITY ||
-		cell_to_v_store_[3 * t + 1] == VERTEX_AT_INFINITY ||
-		cell_to_v_store_[3 * t + 2] == VERTEX_AT_INFINITY
-	    );
+                !triangle_is_free(t) && (
+                    cell_to_v_store_[3 * t] == VERTEX_AT_INFINITY ||
+                    cell_to_v_store_[3 * t + 1] == VERTEX_AT_INFINITY ||
+                    cell_to_v_store_[3 * t + 2] == VERTEX_AT_INFINITY
+                );
         }
 
         /**
@@ -554,7 +554,7 @@ namespace GEO {
          * \return the index of the newly created triangle
          */
         index_t new_triangle(
-            signed_index_t v1, signed_index_t v2, 
+            signed_index_t v1, signed_index_t v2,
             signed_index_t v3
         ) {
             index_t result = new_triangle();
@@ -566,7 +566,7 @@ namespace GEO {
 
         /**
          * \brief Generates a unique stamp for marking tets.
-         * \details Storage is shared for list-chaining and stamp-marking 
+         * \details Storage is shared for list-chaining and stamp-marking
          * (both are mutually exclusive), therefore the stamp has
          * the NOT_IN_LIST_BIT set.
          * \param[in] stamp the unique stamp for marking tets
@@ -599,7 +599,7 @@ namespace GEO {
          * \details A triangle is marked whenever it is
          *  detected as non-conflict. The same space is used
          *  for marking and for chaining the conflict list.
-         *  The index of the point being inserted is used as a 
+         *  The index of the point being inserted is used as a
          *  time-stamp for marking triangles.
          *  A triangle can be in the following states:
          *  - in list
@@ -614,7 +614,7 @@ namespace GEO {
         // _________ Combinatorics ___________________________________
 
         /**
-         * \brief Returns the local index of a vertex by 
+         * \brief Returns the local index of a vertex by
          *   edge and by local vertex index in the edge.
          * \details
          * tri edge vertex is such that the triangle
@@ -626,7 +626,7 @@ namespace GEO {
          * any vertex lv.
          * \param[in] e local facet index, in (0,1,2)
          * \param[in] v local vertex index, in (0,1)
-         * \return the local triangle vertex index of 
+         * \return the local triangle vertex index of
          *  vertex \p v in edge \p f
          */
         static index_t triangle_edge_vertex(index_t e, index_t v) {
@@ -670,7 +670,7 @@ namespace GEO {
          * \return the global index of the \p lv%th vertex of triangle \p t
          * \pre Vertex \p lv of triangle \p t is not at infinity
          */
-         index_t finite_triangle_vertex(index_t t, index_t lv) const {
+        index_t finite_triangle_vertex(index_t t, index_t lv) const {
             geo_debug_assert(t < max_t());
             geo_debug_assert(lv < 3);
             geo_debug_assert(cell_to_v_store_[3 * t + lv] != -1);
@@ -713,12 +713,12 @@ namespace GEO {
             geo_debug_assert(t1 < max_t());
             geo_debug_assert(t2 < max_t());
             geo_debug_assert(le1 < 3);
-	    geo_debug_assert(t1 != t2);
+            geo_debug_assert(t1 != t2);
             cell_to_cell_store_[3 * t1 + le1] = signed_index_t(t2);
         }
-        
+
         /**
-         * \brief Finds the index of the edge accros which t1 is 
+         * \brief Finds the index of the edge accros which t1 is
          *  adjacent to t2_in.
          * \param[in] t1 first triangle
          * \param[in] t2_in second triangle
@@ -761,7 +761,7 @@ namespace GEO {
         void set_tet(
             index_t t,
             signed_index_t v0, signed_index_t v1,
-            signed_index_t v2, 
+            signed_index_t v2,
             index_t a0, index_t a1, index_t a2
         ) {
             geo_debug_assert(t < max_t());
@@ -854,7 +854,7 @@ namespace GEO {
                 double h = heights_[pindex];
                 return (PCK::orient_2dlifted_SOS(
                             pv[0],pv[1],pv[2],p,h0,h1,h2,h
-                       ) > 0) ;
+                        ) > 0) ;
             }
 
             return (PCK::in_circle_2d_SOS(pv[0], pv[1], pv[2], p) > 0);
@@ -874,16 +874,16 @@ namespace GEO {
             const signed_index_t* T, signed_index_t v
         ) {
             // The following expression is 10% faster than using
-            // if() statements. This uses the C++ norm, that 
-            // ensures that the 'true' boolean value converted to 
-            // an int is always 1. With most compilers, this avoids 
+            // if() statements. This uses the C++ norm, that
+            // ensures that the 'true' boolean value converted to
+            // an int is always 1. With most compilers, this avoids
             // generating branching instructions.
             // Thank to Laurent Alonso for this idea.
             index_t result = index_t( (T[1] == v) | ((T[2] == v) * 2) );
             // Sanity check, important if it was T[0], not explicitly
             // tested (detects input that does not meet the precondition).
             geo_debug_assert(T[result] == v);
-            return result; 
+            return result;
         }
 
         /**
@@ -937,17 +937,17 @@ namespace GEO {
         /**
          * Performs additional checks (costly !)
          */
-         bool debug_mode_;
+        bool debug_mode_;
 
         /**
          * Displays the result of the additional checks.
          */
-         bool verbose_debug_mode_;
+        bool verbose_debug_mode_;
 
         /**
          * Displays the timing of the core algorithm.
          */
-         bool benchmark_mode_;
+        bool benchmark_mode_;
 
         /**
          * \brief Gives the indexing of triangle edge
@@ -957,23 +957,23 @@ namespace GEO {
          *  local edge index le (in 0,1,2) and a
          *  local vertex index within the edge (in 0,1).
          */
-	 static char triangle_edge_vertex_[3][2];
-
-	 /**
-	  * \brief Used by find_conflict_zone_iterative()
-	  */
-	 std::stack<index_t> S_;
-
-	/**
-	 * \brief Regular triangulations can have empty cells.
-	 */
-	 bool has_empty_cells_;
+        static char triangle_edge_vertex_[3][2];
 
         /**
-	 * \brief Stop inserting points as soon as an empty cell 
-	 *  is encountered.
-	 */
-	 bool abort_if_empty_cell_;
+         * \brief Used by find_conflict_zone_iterative()
+         */
+        std::stack<index_t> S_;
+
+        /**
+         * \brief Regular triangulations can have empty cells.
+         */
+        bool has_empty_cells_;
+
+        /**
+         * \brief Stop inserting points as soon as an empty cell
+         *  is encountered.
+         */
+        bool abort_if_empty_cell_;
     };
 
     /************************************************************************/
@@ -1011,4 +1011,3 @@ namespace GEO {
 }
 
 #endif
-
