@@ -13,7 +13,7 @@
  *  * Neither the name of the ALICE Project-Team nor the names of its
  *  contributors may be used to endorse or promote products derived from this
  *  software without specific prior written permission.
- * 
+ *
  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -49,8 +49,6 @@
 #include <geogram/basic/algorithm.h>
 #include <geogram/bibliography/bibliography.h>
 
-#include <random>
-
 namespace {
 
     using namespace GEO;
@@ -140,7 +138,7 @@ namespace {
         }
         VertexArray vertices;
     };
-    
+
     /************************************************************************/
 
     /**
@@ -264,7 +262,7 @@ namespace {
     /************************************************************************/
 
 #ifndef GEOGRAM_PSM
-    
+
     /**
      * \brief Base class for facets ordering.
      * \tparam COORD the coordinate to compare
@@ -289,11 +287,11 @@ namespace {
          */
         double center(index_t f) const {
             double result = 0.0;
-	    double s = 1.0 / double(mesh_.facets.nb_vertices(f));
+            double s = 1.0 / double(mesh_.facets.nb_vertices(f));
             for(index_t c: mesh_.facets.corners(f)) {
                 result += s*mesh_.vertices.point_ptr(
                     mesh_.facet_corners.vertex(c)
-		)[COORD]; 
+                )[COORD];
             }
             return result;
         }
@@ -549,7 +547,7 @@ namespace {
     };
 
     /************************************************************************/
-    
+
     /**
      * \brief Base class for cells ordering.
      * \tparam COORD the coordinate to compare
@@ -691,15 +689,15 @@ namespace {
     };
 
 #endif
-    
+
     /************************************************************************/
-    
+
     /**
      * \brief Generic class for sorting arbitrary elements in
      *  Hilbert and Morton orders in 3d.
      * \details The implementation is inspired by:
-     *  - Christophe Delage and Olivier Devillers. Spatial Sorting. 
-     *   In CGAL User and Reference Manual. CGAL Editorial Board, 
+     *  - Christophe Delage and Olivier Devillers. Spatial Sorting.
+     *   In CGAL User and Reference Manual. CGAL Editorial Board,
      *   3.9 edition, 2011
      * \tparam CMP the comparator class for ordering the elements. CMP
      *  is itself a template parameterized by~:
@@ -760,13 +758,13 @@ namespace {
 
         /**
          * \brief Sorts a sequence of elements spatially.
-         * \details This function does an indirect sort, 
-         *  in the sense that a sequence 
-         *  of indices that refer to the elements is sorted. 
+         * \details This function does an indirect sort,
+         *  in the sense that a sequence
+         *  of indices that refer to the elements is sorted.
          *  This function uses a multithreaded implementation.
          * \param[in] M the mesh in which the elements to sort reside
          * \param[in] b an iterator to the first index to be sorted
-         * \param[in] e an iterator one position past the last index 
+         * \param[in] e an iterator one position past the last index
          *  to be sorted
          * \param[in] limit subsequences smaller than limit are left unsorted
          */
@@ -777,78 +775,78 @@ namespace {
             index_t limit = 1
         ) :
             M_(M)
-        {
-            geo_debug_assert(e >= b);
-	    geo_cite_with_info(
-		"WEB:SpatialSorting",
-		"The implementation of spatial sort in GEOGRAM is inspired by "
-		"the idea of using \\verb|std::nth_element()| and the recursive"
-                " template in the spatial sort package of CGAL"
-	    );
+            {
+                geo_debug_assert(e >= b);
+                geo_cite_with_info(
+                    "WEB:SpatialSorting",
+                    "The implementation of spatial sort in GEOGRAM is inspired by "
+                    "the idea of using \\verb|std::nth_element()| and the recursive"
+                    " template in the spatial sort package of CGAL"
+                );
 
-            // If the sequence is smaller than the limit, skip it
-            if(index_t(e - b) <= limit) {
-                return;
-            }
+                // If the sequence is smaller than the limit, skip it
+                if(index_t(e - b) <= limit) {
+                    return;
+                }
 
-            // If the sequence is smaller than 1024, use sequential sorting
-            if(index_t(e - b) < 1024) {
-                sort<0, false, false, false>(M_, b, e);
-                return;
-            }
+                // If the sequence is smaller than 1024, use sequential sorting
+                if(index_t(e - b) < 1024) {
+                    sort<0, false, false, false>(M_, b, e);
+                    return;
+                }
 
-            // Parallel sorting (2 then 4 then 8 sorts in parallel)
+                // Parallel sorting (2 then 4 then 8 sorts in parallel)
 
 // Unfortunately we cannot access consts for template arguments in lambdas in all
 // compilers (gcc is OK but not MSVC) so I'm using (ugly) macros here...
-	    
+
 #          define COORDX 0
 #          define COORDY 1
 #          define COORDZ 2
 #          define UPX false
 #          define UPY false
 #          define UPZ false
-	    
-            m0_ = b;
-            m8_ = e;
-            m4_ = reorder_split(m0_, m8_, CMP<COORDX, UPX, MESH>(M));
 
-	    
-	    parallel(
-		[this]() { m2_ = reorder_split(m0_, m4_, CMP<COORDY,  UPY, MESH>(M_)); },
-		[this]() { m6_ = reorder_split(m4_, m8_, CMP<COORDY, !UPY, MESH>(M_)); }
-	    );
+                m0_ = b;
+                m8_ = e;
+                m4_ = reorder_split(m0_, m8_, CMP<COORDX, UPX, MESH>(M));
 
-	    parallel(
-		[this]() { m1_ = reorder_split(m0_, m2_, CMP<COORDZ,  UPZ, MESH>(M_)); },
-		[this]() { m3_ = reorder_split(m2_, m4_, CMP<COORDZ, !UPZ, MESH>(M_)); },
-		[this]() { m5_ = reorder_split(m4_, m6_, CMP<COORDZ,  UPZ, MESH>(M_)); },
-		[this]() { m7_ = reorder_split(m6_, m8_, CMP<COORDZ, !UPZ, MESH>(M_)); }
-	    );
 
-	    parallel(
-		[this]() { sort<COORDZ,  UPZ,  UPX,  UPY>(M_, m0_, m1_); },
-		[this]() { sort<COORDY,  UPY,  UPZ,  UPX>(M_, m1_, m2_); },
-		[this]() { sort<COORDY,  UPY,  UPZ,  UPX>(M_, m2_, m3_); },
-		[this]() { sort<COORDX,  UPX, !UPY, !UPZ>(M_, m3_, m4_); },
-		[this]() { sort<COORDX,  UPX, !UPY, !UPZ>(M_, m4_, m5_); },
-		[this]() { sort<COORDY, !UPY,  UPZ, !UPX>(M_, m5_, m6_); },
-		[this]() { sort<COORDY, !UPY,  UPZ, !UPX>(M_, m6_, m7_); },
-		[this]() { sort<COORDZ, !UPZ, !UPX,  UPY>(M_, m7_, m8_); }
-	    );
+                parallel(
+                    [this]() { m2_ = reorder_split(m0_, m4_, CMP<COORDY,  UPY, MESH>(M_)); },
+                    [this]() { m6_ = reorder_split(m4_, m8_, CMP<COORDY, !UPY, MESH>(M_)); }
+                );
+
+                parallel(
+                    [this]() { m1_ = reorder_split(m0_, m2_, CMP<COORDZ,  UPZ, MESH>(M_)); },
+                    [this]() { m3_ = reorder_split(m2_, m4_, CMP<COORDZ, !UPZ, MESH>(M_)); },
+                    [this]() { m5_ = reorder_split(m4_, m6_, CMP<COORDZ,  UPZ, MESH>(M_)); },
+                    [this]() { m7_ = reorder_split(m6_, m8_, CMP<COORDZ, !UPZ, MESH>(M_)); }
+                );
+
+                parallel(
+                    [this]() { sort<COORDZ,  UPZ,  UPX,  UPY>(M_, m0_, m1_); },
+                    [this]() { sort<COORDY,  UPY,  UPZ,  UPX>(M_, m1_, m2_); },
+                    [this]() { sort<COORDY,  UPY,  UPZ,  UPX>(M_, m2_, m3_); },
+                    [this]() { sort<COORDX,  UPX, !UPY, !UPZ>(M_, m3_, m4_); },
+                    [this]() { sort<COORDX,  UPX, !UPY, !UPZ>(M_, m4_, m5_); },
+                    [this]() { sort<COORDY, !UPY,  UPZ, !UPX>(M_, m5_, m6_); },
+                    [this]() { sort<COORDY, !UPY,  UPZ, !UPX>(M_, m6_, m7_); },
+                    [this]() { sort<COORDZ, !UPZ, !UPX,  UPY>(M_, m7_, m8_); }
+                );
 
 #          undef COORDX
 #          undef COORDY
 #          undef COORDZ
 #          undef UPX
 #          undef UPY
-#          undef UPZ	    
-        }
+#          undef UPZ
+            }
 
     private:
         const MESH& M_;
         vector<index_t>::iterator
-            m0_, m1_, m2_, m3_, m4_, m5_, m6_, m7_, m8_;
+        m0_, m1_, m2_, m3_, m4_, m5_, m6_, m7_, m8_;
     };
 
     /************************************************************************/
@@ -857,8 +855,8 @@ namespace {
      * \brief Generic class for sorting arbitrary elements in
      *  Hilbert and Morton orders in 3d.
      * \details The implementation is inspired by:
-     *  - Christophe Delage and Olivier Devillers. Spatial Sorting. 
-     *   In CGAL User and Reference Manual. CGAL Editorial Board, 
+     *  - Christophe Delage and Olivier Devillers. Spatial Sorting.
+     *   In CGAL User and Reference Manual. CGAL Editorial Board,
      *   3.9 edition, 2011
      * \tparam CMP the comparator class for ordering the elements. CMP
      *  is itself a template parameterized by~:
@@ -882,7 +880,7 @@ namespace {
          *  last element of the sequence
          * \param[in] limit subsequences smaller than limit are left unsorted
          * \tparam COORDX the first coordinate, can be 0,1 or 2. The second
-         *  coordinate is COORDX+1 modulo 2. 
+         *  coordinate is COORDX+1 modulo 2.
          * \tparam UPX whether ordering along the first coordinate
          *  is direct or inverse
          * \tparam UPY whether ordering along the second coordinate
@@ -896,27 +894,27 @@ namespace {
             if(end - begin <= signed_index_t(limit)) {
                 return;
             }
-	    IT m0 = begin, m4 = end;
+            IT m0 = begin, m4 = end;
 
-	    IT m2 = reorder_split (m0, m4, CMP<COORDX,  UPX, MESH>(M));
-	    IT m1 = reorder_split (m0, m2, CMP<COORDY,  UPY, MESH>(M));
-	    IT m3 = reorder_split (m2, m4, CMP<COORDY, !UPY, MESH>(M));
+            IT m2 = reorder_split (m0, m4, CMP<COORDX,  UPX, MESH>(M));
+            IT m1 = reorder_split (m0, m2, CMP<COORDY,  UPY, MESH>(M));
+            IT m3 = reorder_split (m2, m4, CMP<COORDY, !UPY, MESH>(M));
 
-	    sort<COORDY, UPY, UPX> (M, m0, m1);
-	    sort<COORDX, UPX, UPY> (M, m1, m2);
-	    sort<COORDX, UPX, UPY> (M, m2, m3);
-	    sort<COORDY,!UPY,!UPX> (M, m3, m4);
+            sort<COORDY, UPY, UPX> (M, m0, m1);
+            sort<COORDX, UPX, UPY> (M, m1, m2);
+            sort<COORDX, UPX, UPY> (M, m2, m3);
+            sort<COORDY,!UPY,!UPX> (M, m3, m4);
         }
 
         /**
          * \brief Sorts a sequence of elements spatially.
-         * \details This function does an indirect sort, 
-         *  in the sense that a sequence 
-         *  of indices that refer to the elements is sorted. 
+         * \details This function does an indirect sort,
+         *  in the sense that a sequence
+         *  of indices that refer to the elements is sorted.
          *  This function uses a multithreaded implementation.
          * \param[in] M the mesh in which the elements to sort reside
          * \param[in] b an iterator to the first index to be sorted
-         * \param[in] e an iterator one position past the last index 
+         * \param[in] e an iterator one position past the last index
          *  to be sorted
          * \param[in] limit subsequences smaller than limit are left unsorted
          */
@@ -927,21 +925,21 @@ namespace {
             index_t limit = 1
         ) :
             M_(M)
-        {
-            geo_debug_assert(e > b);
-	    geo_cite_with_info(
-		"WEB:SpatialSorting",
-		"The implementation of spatial sort in GEOGRAM is inspired by "
-		"the idea of using \\verb|std::nth_element()| and the recursive"
-                " template in the spatial sort package of CGAL"
-	    );
+            {
+                geo_debug_assert(e > b);
+                geo_cite_with_info(
+                    "WEB:SpatialSorting",
+                    "The implementation of spatial sort in GEOGRAM is inspired by "
+                    "the idea of using \\verb|std::nth_element()| and the recursive"
+                    " template in the spatial sort package of CGAL"
+                );
 
-            // If the sequence is smaller than the limit, skip it
-            if(index_t(e - b) <= limit) {
-                return;
+                // If the sequence is smaller than the limit, skip it
+                if(index_t(e - b) <= limit) {
+                    return;
+                }
+                sort<0, false, false>(M_, b, e);
             }
-	    sort<0, false, false>(M_, b, e);
-        }
     private:
         const MESH& M_;
     };
@@ -949,7 +947,7 @@ namespace {
     /************************************************************************/
 
 #ifndef GEOGRAM_PSM
-    
+
     /**
      * \brief Sorts the vertices of a mesh according to the Hilbert ordering.
      * \details The function does not change the mesh, it computes instead
@@ -958,8 +956,8 @@ namespace {
      *  attributes).
      * \param[in] M the mesh where the vertices to be sorted reside
      * \param[out] sorted_indices the permutation to be applied
-       to the vertices
-     */
+     to the vertices
+    */
     void hilbert_vsort_3d(
         const Mesh& M, vector<index_t>& sorted_indices
     ) {
@@ -1090,11 +1088,11 @@ namespace {
         }
     }
 
-#endif    
-    
+#endif
+
     /**
      * \brief Computes the BRIO order for a set of 3D points.
-     * \details Implementation of compute_BRIO_order(). 
+     * \details Implementation of compute_BRIO_order().
      *  It is used to accelerate incremental insertion in Delaunay triangulation
      * \param[in] nb_vertices number of vertices to sort
      * \param[in] vertices pointer to the coordinates of the vertices
@@ -1124,10 +1122,10 @@ namespace {
         vector<index_t>::iterator m = b;
         if(index_t(e - b) > threshold) {
             ++depth;
-            m = b + int(double(e - b) * ratio);
+            m = b + signed_index_t(double(e - b) * ratio);
             compute_BRIO_order_recursive(
                 nb_vertices, vertices,
-		dimension, stride,
+                dimension, stride,
                 sorted_indices, b, m,
                 threshold, ratio, depth,
                 levels
@@ -1135,17 +1133,17 @@ namespace {
         }
 
         VertexMesh M(nb_vertices, vertices, stride);
-	if(dimension == 3) {
-	    HilbertSort3d<Hilbert_vcmp, VertexMesh>(
-		M, m, e
-	    );
-	} else if(dimension ==2) {
-	    HilbertSort2d<Hilbert_vcmp, VertexMesh>(
-		M, m, e
-	    );
-	} else {
-	    geo_assert_not_reached;
-	}
+        if(dimension == 3) {
+            HilbertSort3d<Hilbert_vcmp, VertexMesh>(
+                M, m, e
+            );
+        } else if(dimension ==2) {
+            HilbertSort2d<Hilbert_vcmp, VertexMesh>(
+                M, m, e
+            );
+        } else {
+            geo_assert_not_reached;
+        }
 
         if(levels != nullptr) {
             levels->push_back(index_t(e - sorted_indices.begin()));
@@ -1158,7 +1156,7 @@ namespace {
 namespace GEO {
 
 #ifndef GEOGRAM_PSM
-    
+
     void mesh_reorder(Mesh& M, MeshOrder order) {
 
         geo_assert(M.vertices.dimension() >= 3);
@@ -1167,12 +1165,12 @@ namespace GEO {
         {
             vector<index_t> sorted_indices;
             switch(order) {
-                case MESH_ORDER_HILBERT:
-                    hilbert_vsort_3d(M, sorted_indices);
-                    break;
-                case MESH_ORDER_MORTON:
-                    morton_vsort_3d(M, sorted_indices);
-                    break;
+            case MESH_ORDER_HILBERT:
+                hilbert_vsort_3d(M, sorted_indices);
+                break;
+            case MESH_ORDER_MORTON:
+                morton_vsort_3d(M, sorted_indices);
+                break;
             }
             M.vertices.permute_elements(sorted_indices);
         }
@@ -1181,12 +1179,12 @@ namespace GEO {
         if(M.facets.nb() != 0) {
             vector<index_t> sorted_indices;
             switch(order) {
-                case MESH_ORDER_HILBERT:
-                    hilbert_fsort_3d(M, sorted_indices);
-                    break;
-                case MESH_ORDER_MORTON:
-                    morton_fsort_3d(M, sorted_indices);
-                    break;
+            case MESH_ORDER_HILBERT:
+                hilbert_fsort_3d(M, sorted_indices);
+                break;
+            case MESH_ORDER_MORTON:
+                morton_fsort_3d(M, sorted_indices);
+                break;
             }
             M.facets.permute_elements(sorted_indices);
         }
@@ -1195,12 +1193,12 @@ namespace GEO {
         if(M.cells.nb() != 0) {
             vector<index_t> sorted_indices;
             switch(order) {
-                case MESH_ORDER_HILBERT:
-                    hilbert_csort_3d(M, sorted_indices);
-                    break;
-                case MESH_ORDER_MORTON:
-                    morton_csort_3d(M, sorted_indices);
-                    break;
+            case MESH_ORDER_HILBERT:
+                hilbert_csort_3d(M, sorted_indices);
+                break;
+            case MESH_ORDER_MORTON:
+                morton_csort_3d(M, sorted_indices);
+                break;
             }
             M.cells.permute_elements(sorted_indices);
         }
@@ -1221,25 +1219,25 @@ namespace GEO {
             return;
         }
         VertexMesh M(total_nb_vertices, vertices, stride);
-	if(dimension == 3) {
-	    HilbertSort3d<Hilbert_vcmp, VertexMesh>(
-		M, sorted_indices.begin() + int(first),
-		sorted_indices.begin() + int(last)
-	    );
-	} else if(dimension == 2) {
-	    HilbertSort2d<Hilbert_vcmp, VertexMesh>(
-		M, sorted_indices.begin() + int(first),
-		sorted_indices.begin() + int(last)
-	    );
-	} else {
-	    geo_assert_not_reached;
-	}
+        if(dimension == 3) {
+            HilbertSort3d<Hilbert_vcmp, VertexMesh>(
+                M, sorted_indices.begin() + int(first),
+                sorted_indices.begin() + int(last)
+            );
+        } else if(dimension == 2) {
+            HilbertSort2d<Hilbert_vcmp, VertexMesh>(
+                M, sorted_indices.begin() + int(first),
+                sorted_indices.begin() + int(last)
+            );
+        } else {
+            geo_assert_not_reached;
+        }
     }
-    
+
     void compute_BRIO_order(
         index_t nb_vertices, const double* vertices,
         vector<index_t>& sorted_indices,
-	index_t dimension,
+        index_t dimension,
         index_t stride,
         index_t threshold,
         double ratio,
@@ -1255,17 +1253,11 @@ namespace GEO {
             sorted_indices[i] = i;
         }
 
-        //The next three lines replace the following commented-out line
-        //(random_shuffle is deprecated in C++17, and they call this 
-        // progess...)
-        //std::random_shuffle(sorted_indices.begin(), sorted_indices.end()); 
-        std::random_device rng;
-        std::mt19937 urng(rng());
-        std::shuffle(sorted_indices.begin(), sorted_indices.end(), urng);
+	GEO::random_shuffle(sorted_indices.begin(), sorted_indices.end());
 
         compute_BRIO_order_recursive(
             nb_vertices, vertices,
-	    dimension, stride,
+            dimension, stride,
             sorted_indices,
             sorted_indices.begin(), sorted_indices.end(),
             threshold, ratio, depth, levels
@@ -1282,37 +1274,37 @@ namespace {
     // copied here for now because linker does not find
     // it under Android.
     int Periodic_translation[27][3] = {
-	{  0,  0,  0}, //13 -> 0   +   <-- zero displacement is first.
-	{ -1, -1, -1}, //0  -> 1   -
-	{ -1, -1,  0}, //1  -> 2   -
-	{ -1, -1,  1}, //2  -> 3   -
-	{ -1,  0, -1}, //3  -> 4   -
-	{ -1,  0,  0}, //4  -> 5   - 
-	{ -1,  0,  1}, //5  -> 6   -
-	{ -1,  1, -1}, //6  -> 7   -
-	{ -1,  1,  0}, //7  -> 8   -
-	{ -1,  1,  1}, //8  -> 9   - 
-	{  0, -1, -1}, //9  -> 10  -
-	{  0, -1,  0}, //10 -> 11  -
-	{  0, -1,  1}, //11 -> 12  -
-	{  0,  0, -1}, //12 -> 13  -
-	// (zero displacement was there)
-	{  0,  0,  1}, //14 -> 14  +
-	{  0,  1, -1}, //15 -> 15  -
-	{  0,  1,  0}, //16 -> 16  +
-	{  0,  1,  1}, //17 -> 17  +
-	{  1, -1, -1}, //18 -> 18  -
-	{  1, -1,  0}, //19 -> 19  -
-	{  1, -1,  1}, //20 -> 20  -
-	{  1,  0, -1}, //21 -> 21  -
-	{  1,  0,  0}, //22 -> 22  +
-	{  1,  0,  1}, //23 -> 23  +
-	{  1,  1, -1}, //24 -> 24  -
-	{  1,  1,  0}, //25 -> 25  +
-	{  1,  1,  1}  //26 -> 26  +
+        {  0,  0,  0}, //13 -> 0   +   <-- zero displacement is first.
+        { -1, -1, -1}, //0  -> 1   -
+        { -1, -1,  0}, //1  -> 2   -
+        { -1, -1,  1}, //2  -> 3   -
+        { -1,  0, -1}, //3  -> 4   -
+        { -1,  0,  0}, //4  -> 5   -
+        { -1,  0,  1}, //5  -> 6   -
+        { -1,  1, -1}, //6  -> 7   -
+        { -1,  1,  0}, //7  -> 8   -
+        { -1,  1,  1}, //8  -> 9   -
+        {  0, -1, -1}, //9  -> 10  -
+        {  0, -1,  0}, //10 -> 11  -
+        {  0, -1,  1}, //11 -> 12  -
+        {  0,  0, -1}, //12 -> 13  -
+        // (zero displacement was there)
+        {  0,  0,  1}, //14 -> 14  +
+        {  0,  1, -1}, //15 -> 15  -
+        {  0,  1,  0}, //16 -> 16  +
+        {  0,  1,  1}, //17 -> 17  +
+        {  1, -1, -1}, //18 -> 18  -
+        {  1, -1,  0}, //19 -> 19  -
+        {  1, -1,  1}, //20 -> 20  -
+        {  1,  0, -1}, //21 -> 21  -
+        {  1,  0,  0}, //22 -> 22  +
+        {  1,  0,  1}, //23 -> 23  +
+        {  1,  1, -1}, //24 -> 24  -
+        {  1,  1,  0}, //25 -> 25  +
+        {  1,  1,  1}  //26 -> 26  +
     };
 
-    
+
     /**
      * \details Exposes an interface compatible with the requirement
      *   of Hilbert sort templates for a raw array of vertices.
@@ -1321,12 +1313,12 @@ namespace {
     public:
         /**
          * \brief Constructs a new PeriodicVertexArray.
-	 * \param[in] nb_vertices total number of vertices, including
-	 *   the 27 copies.
+         * \param[in] nb_vertices total number of vertices, including
+         *   the 27 copies.
          * \param[in] base address of the points.
          * \param[in] stride number of doubles between
          *  two consecutive points.
-	 * \param[in] period the edge length of the periodic domain.
+         * \param[in] period the edge length of the periodic domain.
          */
         PeriodicVertexArray3d(
             index_t nb_vertices,
@@ -1336,26 +1328,26 @@ namespace {
             base_(base),
             stride_(stride) {
             nb_vertices_ = nb_vertices;
-	    nb_real_vertices_ = nb_vertices_ / 27;
-	    geo_debug_assert(nb_vertices % 27 == 0);
+            nb_real_vertices_ = nb_vertices_ / 27;
+            geo_debug_assert(nb_vertices % 27 == 0);
 
-	    
-	    for(index_t i=0; i<27; ++i) {
-		for(index_t j=0; j<3; ++j) {
-		    xlat_[i][j] = period[j] * double(Periodic_translation[i][j]);
-		}
-	    }
+
+            for(index_t i=0; i<27; ++i) {
+                for(index_t j=0; j<3; ++j) {
+                    xlat_[i][j] = period[j] * double(Periodic_translation[i][j]);
+                }
+            }
         }
 
         /**
          * \brief Gets a point coordinate by its index and coordinate.
          * \param[in] i the index of the point.
-	 * \param[in] coord the coordinate.
+         * \param[in] coord the coordinate.
          * \return the value of the coordinate.
          */
-	double point_coord(index_t i, index_t coord) const {
-	    index_t instance = i / nb_real_vertices_;
-	    i = i % nb_real_vertices_;
+        double point_coord(index_t i, index_t coord) const {
+            index_t instance = i / nb_real_vertices_;
+            i = i % nb_real_vertices_;
             return (base_ + i * stride_)[coord] + xlat_[instance][coord];
         }
 
@@ -1363,8 +1355,8 @@ namespace {
         const double* base_;
         index_t stride_;
         index_t nb_vertices_;
-	index_t nb_real_vertices_;
-	double xlat_[27][3];
+        index_t nb_real_vertices_;
+        double xlat_[27][3];
     };
 
     /**
@@ -1375,8 +1367,8 @@ namespace {
     public:
         /**
          * \brief Constructs a new VertexMesh.
-	 * \param[in] nb_vertices total number of vertices, including
-	 *   the 27 copies.
+         * \param[in] nb_vertices total number of vertices, including
+         *   the 27 copies.
          * \param[in] base address of the points
          * \param[in] stride number of doubles between
          *  two consecutive points
@@ -1384,9 +1376,9 @@ namespace {
         PeriodicVertexMesh3d(
             index_t nb_vertices,
             const double* base, index_t stride, const vec3& period
-	) : vertices(nb_vertices, base, stride, period) {
+        ) : vertices(nb_vertices, base, stride, period) {
         }
-	
+
         PeriodicVertexArray3d vertices;
     };
 
@@ -1437,37 +1429,29 @@ namespace {
         }
         const PeriodicVertexMesh3d& mesh_;
     };
-    
+
 }
 
 namespace GEO {
 
     void Hilbert_sort_periodic(
-	index_t nb_vertices, const double* vertices,
-	vector<index_t>& sorted_indices,
-	index_t dimension,
+        index_t nb_vertices, const double* vertices,
+        vector<index_t>& sorted_indices,
+        index_t dimension,
         index_t stride,
-	vector<index_t>::iterator b,
-	vector<index_t>::iterator e,
-	const vec3& period
+        vector<index_t>::iterator b,
+        vector<index_t>::iterator e,
+        const vec3& period
     ) {
-	geo_assert(dimension == 3); // Only implemented for 3D.	
-	geo_argused(sorted_indices); // Accessed through b and e.
+        geo_assert(dimension == 3); // Only implemented for 3D.
+        geo_argused(sorted_indices); // Accessed through b and e.
 
-       
-        //The next three lines replace the following commented-out line
-        //(random_shuffle is deprecated in C++17, and they call this 
-        // progress...)
-        // std::random_shuffle(b,e);
-        std::random_device rng;
-        std::mt19937 urng(rng());
-        std::shuffle(b,e, urng);
+	GEO::random_shuffle(b,e);
 
-	PeriodicVertexMesh3d M(nb_vertices, vertices, stride, period);
-	HilbertSort3d<Hilbert_vcmp_periodic, PeriodicVertexMesh3d>(
-	    M, b, e
-	);
+        PeriodicVertexMesh3d M(nb_vertices, vertices, stride, period);
+        HilbertSort3d<Hilbert_vcmp_periodic, PeriodicVertexMesh3d>(
+            M, b, e
+        );
     }
 
 }
-
