@@ -13,7 +13,7 @@
  *  * Neither the name of the ALICE Project-Team nor the names of its
  *  contributors may be used to endorse or promote products derived from this
  *  software without specific prior written permission.
- * 
+ *
  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -47,53 +47,32 @@
 #endif
 
 #include <geogram/numerics/multi_precision.h>
+#include <geogram/numerics/PCK.h>
 #include <geogram/basic/process.h>
+#include <geogram/basic/logger.h>
 
 namespace {
 
     using namespace GEO;
 
-    /************************************************************************/
-    
-    bool expansion_length_stat_ = false;
+#ifdef PCK_STATS
     std::vector<index_t> expansion_length_histo_;
-
-    /**
-     * \brief Displays statistics about expansions allocation
-     * \details An instance of the class is declared as a static variable.
-     * The statistics are automatically printed when it gets deleted when the
-     * program exits normally.
-     */
-    class ExpansionStatsDisplay {
-    public:
-        /**
-         * \brief ExpansionStatsDisplay destructor.
-         * \details Displays the statistics on exit.
-         */
-        ~ExpansionStatsDisplay() {
-            for(index_t i = 0; i < expansion_length_histo_.size(); ++i) {
-                std::cerr << "expansion len " << i
-                    << " : " << expansion_length_histo_[i] << std::endl;
-            }
-        }
-    };
-
-    ExpansionStatsDisplay expansion_stats_display_;
+#endif
 
     /************************************************************************/
 
     /**
-     * \brief An optimized memory allocator for objects of 
+     * \brief An optimized memory allocator for objects of
      *  small size.
      * \details It is used by the high-level class expansion_nt
      *  that allocates expansion objects on the heap. PCK predicates
-     *  do not use it (they use the more efficient low-level API 
+     *  do not use it (they use the more efficient low-level API
      *  that allocates expansion objects on the stack).
      */
     class Pools {
 
     public:
-        
+
         /**
          * \brief Creates a new Pools object
          */
@@ -120,7 +99,7 @@ namespace {
         void* malloc(size_t size) {
             if(size >= pools_.size()) {
                 return ::malloc(size);
-            } 
+            }
             if(pools_[size] == nullptr) {
                 new_chunk(size);
             }
@@ -144,17 +123,17 @@ namespace {
             pools_[size] = Memory::pointer(ptr);
         }
 
-        
+
     protected:
         /**
          * \brief Number of elements in each individual chunk
          *  allocation.
          */
         static const index_t NB_ITEMS_PER_CHUNK = 512;
-        
+
         /**
          * \brief Allocates a new chunk of elements and prepends
-         *  it to the free list for allocations of the specified 
+         *  it to the free list for allocations of the specified
          *  size.
          * \param[in] item_size size of the elements to be allocated.
          */
@@ -202,7 +181,7 @@ namespace {
         }
 
         /**
-         * \brief Gets a pointer to an item by chunk, item_size 
+         * \brief Gets a pointer to an item by chunk, item_size
          *   and index
          * \pre index < NB_ITEMS_IN_CHUNK
          * \param[in] chunk a pointer to the chunk
@@ -216,13 +195,13 @@ namespace {
             geo_debug_assert(index < NB_ITEMS_PER_CHUNK);
             return chunk + (item_size * size_t(index));
         }
-        
+
         /**
          * \brief The free lists of the pools. Index corresponds
          *  to element size in bytes.
          */
         std::vector<Memory::pointer> pools_;
-        
+
         /**
          * \brief Pointers to all the allocated chunks.
          * \details Used by the destructor to release all the
@@ -233,9 +212,9 @@ namespace {
     };
 
     static Pools pools_;
-    
+
     /************************************************************************/
-    
+
     /**
      * \brief Computes the sum of two doubles into a length 2 expansion.
      * \details By Jonathan Shewchuk.
@@ -251,7 +230,7 @@ namespace {
         y = b - bvirt;
     }
 
-#ifdef REMOVE_ME        
+#ifdef REMOVE_ME
     /**
      * \brief Computes the difference of two doubles into a length 2 expansion.
      * \details By Jonathan Shewchuk.
@@ -267,7 +246,7 @@ namespace {
         y = bvirt - b;
     }
 #endif
-    
+
     /**
      * \brief Computes the sum of a length 2 expansion and a double
      *  into a length 3 expansion.
@@ -309,6 +288,8 @@ namespace {
         two_one_sum(_j, _0, b1, x3, x2, x1);
     }
 
+#ifndef FP_FAST_FMA
+
     /**
      * \brief Computes the product between two doubles where
      *  the second one have already been split.
@@ -358,6 +339,8 @@ namespace {
         y = (alo * blo) - err3;
     }
 
+#endif
+
     /**
      * \brief Computes the square of an expansion of length 2.
      * \param[in] a1 high-magnitude component of the argument
@@ -463,6 +446,55 @@ namespace {
         two_sum(_m, _k, x[7], x[6]);
 #endif
     }
+
+    // [Shewchuk 97]
+    // (https://people.eecs.berkeley.edu/~jrs/papers/robustr.pdf)
+    // Section 2.8: other operations
+    // Compression
+    // Note: when converting the algorithms in Shewchuk's article
+    // into code, indices in the article go from 1 to m, and in the
+    // code they go from 0 to m-1 !!!
+    // /!\ there is a bug in the original article,
+    // line 14 of the algorigthm should be h_top <= q (small q and not capital Q)
+
+    /**
+     * \brief Compresses an expansion
+     * \details Modifies in-place an expansion in such a way that it
+     *  is shorter. The represented value is not modified.
+     * \param[in,out] e a reference to the expansion to be compressed
+     */
+    void compress_expansion(expansion& e) {
+        expansion& h = e;
+
+        index_t m = e.length();
+        double Qnew,q;
+
+        index_t bottom = m-1;
+        double Q = e[bottom];
+
+        for(int i=int(m)-2; i>=0; --i) {
+            fast_two_sum(Q, e[index_t(i)], Qnew, q);
+            Q = Qnew;
+            if(q != 0.0) {
+                h[bottom] = Q;
+                --bottom;
+                Q = q;
+            }
+        }
+        h[bottom] = Q;
+
+        index_t top = 0;
+        for(index_t i=bottom+1; i<m; ++i) {
+            fast_two_sum(h[i],Q,Qnew,q);
+            Q = Qnew;
+            if(q != 0) {
+                h[top] = q;
+                ++top;
+            }
+        }
+        h[top] = Q;
+        h.set_length(top+1);
+    }
 }
 
 namespace GEO {
@@ -695,6 +727,7 @@ namespace GEO {
         }
         h.set_length(hindex);
     }
+
 }
 
 /****************************************************************************/
@@ -732,27 +765,27 @@ namespace GEO {
     }
 
     static Process::spinlock expansions_lock = GEOGRAM_SPINLOCK_INIT;
-    
+
     expansion* expansion::new_expansion_on_heap(index_t capa) {
-	Process::acquire_spinlock(expansions_lock);
-        if(expansion_length_stat_) {
-            if(capa >= expansion_length_histo_.size()) {
-                expansion_length_histo_.resize(capa + 1);
-            }
-            expansion_length_histo_[capa]++;
+        Process::acquire_spinlock(expansions_lock);
+#ifdef PCK_STATS
+        if(capa >= expansion_length_histo_.size()) {
+            expansion_length_histo_.resize(capa + 1);
         }
+        expansion_length_histo_[capa]++;
+#endif
         Memory::pointer addr = Memory::pointer(
             pools_.malloc(expansion::bytes(capa))
         );
-	Process::release_spinlock(expansions_lock);
+        Process::release_spinlock(expansions_lock);
         expansion* result = new(addr)expansion(capa);
         return result;
     }
 
     void expansion::delete_expansion_on_heap(expansion* e) {
-	Process::acquire_spinlock(expansions_lock);	
+        Process::acquire_spinlock(expansions_lock);
         pools_.free(e, expansion::bytes(e->capacity()));
-	Process::release_spinlock(expansions_lock);	
+        Process::release_spinlock(expansions_lock);
     }
 
     // ====== Initialization from expansion and double ===============
@@ -827,13 +860,42 @@ namespace GEO {
         } else {
             // "Distillation" (see Shewchuk's paper) is computed recursively,
             // by splitting the list of expansions to sum into two halves.
+
             const double* a1 = a;
             index_t a1_length = a_length / 2;
             const double* a2 = a1 + a1_length;
             index_t a2_length = a_length - a1_length;
-            expansion& a1b = expansion_sub_product(a1, a1_length, b);
-            expansion& a2b = expansion_sub_product(a2, a2_length, b);
-            this->assign_sum(a1b, a2b);
+
+            // Allocate both halves on the stack or on the heap if too large
+            // (some platformes, e.g. MacOSX, have a small stack)
+
+            index_t a1b_capa = sub_product_capacity(a1_length, b.length());
+            index_t a2b_capa = sub_product_capacity(a2_length, b.length());
+
+            bool a1b_on_heap = (a1b_capa > MAX_CAPACITY_ON_STACK);
+            bool a2b_on_heap = (a2b_capa > MAX_CAPACITY_ON_STACK);
+
+            expansion* a1b = a1b_on_heap ?
+                new_expansion_on_heap(a1b_capa) :
+                new_expansion_on_stack(a1b_capa);
+
+            a1b->assign_sub_product(a1, a1_length, b);
+
+            expansion* a2b = a2b_on_heap ?
+                new_expansion_on_heap(a2b_capa) :
+                new_expansion_on_stack(a2b_capa);
+
+            a2b->assign_sub_product(a2, a2_length, b);
+
+            this->assign_sum(*a1b, *a2b);
+
+            if(a1b_on_heap) {
+                delete_expansion_on_heap(a1b);
+            }
+
+            if(a2b_on_heap) {
+                delete_expansion_on_heap(a2b);
+            }
         }
         return *this;
     }
@@ -856,24 +918,84 @@ namespace GEO {
             two_two_product(a.data(), b.data(), x_);
             set_length(8);
         } else {
-            // Recursive distillation: the shortest expansion
-            // is split into two parts.
-            if(a.length() < b.length()) {
-                const double* a1 = a.data();
-                index_t a1_length = a.length() / 2;
-                const double* a2 = a1 + a1_length;
-                index_t a2_length = a.length() - a1_length;
-                expansion& a1b = expansion_sub_product(a1, a1_length, b);
-                expansion& a2b = expansion_sub_product(a2, a2_length, b);
-                this->assign_sum(a1b, a2b);
+
+
+            const expansion* pa = &a;
+            const expansion* pb = &b;
+
+            if(pa->length() > pb->length()) {
+                std::swap(pa, pb);
+            }
+
+            // [Shewchuk 97]
+            // (https://people.eecs.berkeley.edu/~jrs/papers/robustr.pdf)
+            // Section 2.8: other operations
+            // Distillation: sum of k values.
+            //    Worst case: 1/2*k*(k-1)
+            //    But O(k log(k)) if the "summing tree" is well balanced
+            //      and using fast_expansion_sum().
+            // Recommended way of computing a product:
+            //    compute a1*b, a2*b ... ak*b using scale_expansion_zeroelim()
+            //    sum them using a well-balanced tree
+            // However, there is an extra cost for the recursion (and more
+            // importantly, for allocating the intermediary sums, especially
+            // when they do not fit on the stack). So when there are less than
+            // 16 values to add, we simply accumulate them.
+
+            bool use_balanced_distillation = (pa->length() >= 16);
+
+            if(use_balanced_distillation) {
+                // assign_sub_product() is a recursive function that
+                // creates a balanced distillation tree on the stack.
+                assign_sub_product(pa->data(), pa->length(),*pb);
             } else {
-                const double* b1 = b.data();
-                index_t b1_length = b.length() / 2;
-                const double* b2 = b1 + b1_length;
-                index_t b2_length = b.length() - b1_length;
-                expansion& ab1 = expansion_sub_product(b1, b1_length, a);
-                expansion& ab2 = expansion_sub_product(b2, b2_length, a);
-                this->assign_sum(ab1, ab2);
+                // trivial implementation: compute all the products
+                // P = ak*b and accumulate them into S
+
+                index_t P_capa = product_capacity(*pb, 3.0); // 3.0, or any
+                                                             // number that is
+                                                             // not a power of 2
+
+                index_t S_capa = capacity(); // same capacity as this,
+                                             // enough to store sum.
+
+                bool P_on_heap = (P_capa > MAX_CAPACITY_ON_STACK);
+                bool S_on_heap = (S_capa > MAX_CAPACITY_ON_STACK);
+
+                expansion* P = P_on_heap ?
+                    new_expansion_on_heap(P_capa) :
+                    new_expansion_on_stack(P_capa);
+
+                expansion* S = S_on_heap ?
+                    new_expansion_on_heap(S_capa) :
+                    new_expansion_on_stack(S_capa);
+
+                expansion* S1 = S;
+                expansion* S2 = this;
+
+                if((pa->length()%2) == 0) {
+                    std::swap(S1,S2);
+                }
+
+                for(index_t i=0; i<pa->length(); ++i) {
+                    if(i == 0) {
+                        S2->assign_product(*pb, (*pa)[i]);
+                    } else {
+                        P->assign_product(*pb, (*pa)[i]);
+                        S2->assign_sum(*S1,*P);
+                    }
+                    std::swap(S1,S2);
+                }
+
+                geo_assert(S1 == this);
+
+                if(S_on_heap) {
+                    delete_expansion_on_heap(S);
+                }
+
+                if(P_on_heap) {
+                    delete_expansion_on_heap(P);
+                }
             }
         }
         return *this;
@@ -943,7 +1065,7 @@ namespace GEO {
         const double* p1, const double* p2, coord_index_t dim
     ) {
         geo_debug_assert(capacity() >= sq_dist_capacity(dim));
-	geo_debug_assert(dim > 0);
+        geo_debug_assert(dim > 0);
         if(dim == 1) {
             double d0, d1;
             two_diff(p1[0], p2[0], d1, d0);
@@ -987,7 +1109,7 @@ namespace GEO {
             expansion& d1 = expansion_dot_at(p1, p2, p0, dim1);
             expansion& d2 = expansion_dot_at(p1_2, p2_2, p0_2, dim2);
             this->assign_sum(d1, d2);
-        } 
+        }
         return *this;
     }
 
@@ -1000,7 +1122,7 @@ namespace GEO {
         this->assign_sum(x2,y2,z2);
         return *this;
     }
-    
+
     /************************************************************************/
 
     bool expansion::is_same_as(const expansion& rhs) const {
@@ -1023,28 +1145,83 @@ namespace GEO {
     }
 
     Sign expansion::compare(const expansion& rhs) const {
+        // Fast path: different signs or both zero
+        Sign s1 = sign();
+        Sign s2 = rhs.sign();
+        if(s1 == ZERO && s2 == ZERO) {
+            return ZERO;
+        }
+        if(s1 != s2) {
+            return (int(s1) > int(s2) ? POSITIVE : NEGATIVE);
+        }
+
+        // Fast path: same internal representation
         if(is_same_as(rhs)) {
             return ZERO;
         }
+
+        // Compute difference and return sign of difference
+        index_t capa = diff_capacity(*this, rhs);
+        if(capa > MAX_CAPACITY_ON_STACK) {
+            expansion* d = new_expansion_on_heap(capa);
+            d->assign_diff(*this, rhs);
+            Sign result = d->sign();
+            delete_expansion_on_heap(d);
+            return result;
+        }
         const expansion& d = expansion_diff(*this, rhs);
         return d.sign();
     }
-    
+
     Sign expansion::compare(double rhs) const {
-        if(rhs == 0.0) {
-            return sign();
+        // Fast path: different signs or both zero
+        Sign s1 = sign();
+        Sign s2 = geo_sgn(rhs);
+        if(s1 == ZERO && s2 == ZERO) {
+            return ZERO;
         }
+        if(s1 != s2) {
+            return (int(s1) > int(s2) ? POSITIVE : NEGATIVE);
+        }
+
+        // Fast path: same internal representation
         if(is_same_as(rhs)) {
             return ZERO;
         }
+
+        // Compute difference and return sign of difference
+        index_t capa = diff_capacity(*this, rhs);
+        if(capa > MAX_CAPACITY_ON_STACK) {
+            expansion* d = new_expansion_on_heap(capa);
+            d->assign_diff(*this, rhs);
+            Sign result = d->sign();
+            delete_expansion_on_heap(d);
+            return result;
+        }
         const expansion& d = expansion_diff(*this, rhs);
         return d.sign();
     }
-    
+
+
+/************************************************************************/
+
+    void expansion::show_all_stats() {
+#ifdef PCK_STATS
+        Logger::out("expansion") << "Stats" << std::endl;
+        for(index_t i = 0; i < expansion_length_histo_.size(); ++i) {
+            if(expansion_length_histo_[i] != 0) {
+                Logger::out("expansion")
+                    << "len " << i
+                    << " : " << expansion_length_histo_[i] << std::endl;
+            }
+        }
+#endif
+    }
+
     /************************************************************************/
-    
+
     Sign sign_of_expansion_determinant(
-        const expansion& a00,const expansion& a01,  
+        const expansion& a00,const expansion& a01,
         const expansion& a10,const expansion& a11
     ) {
         const expansion& result = expansion_det2x2(a00, a01, a10, a11);
@@ -1058,7 +1235,7 @@ namespace GEO {
     ) {
         // First compute the det2x2
         const expansion& m01 =
-            expansion_det2x2(a00, a10, a01, a11); 
+            expansion_det2x2(a00, a10, a01, a11);
         const expansion& m02 =
             expansion_det2x2(a00, a20, a01, a21);
         const expansion& m12 =
@@ -1072,7 +1249,7 @@ namespace GEO {
         const expansion& result = expansion_sum3(z1,z2,z3);
         return result.sign();
     }
-    
+
     Sign sign_of_expansion_determinant(
         const expansion& a00,const expansion& a01,
         const expansion& a02,const expansion& a03,
@@ -1081,10 +1258,10 @@ namespace GEO {
         const expansion& a20,const expansion& a21,
         const expansion& a22,const expansion& a23,
         const expansion& a30,const expansion& a31,
-        const expansion& a32,const expansion& a33 
+        const expansion& a32,const expansion& a33
     ) {
 
-        // First compute the det2x2        
+        // First compute the det2x2
         const expansion& m01 =
             expansion_det2x2(a10,a00,a11,a01);
         const expansion& m02 =
@@ -1096,8 +1273,8 @@ namespace GEO {
         const expansion& m13 =
             expansion_det2x2(a30,a10,a31,a11);
         const expansion& m23 =
-            expansion_det2x2(a30,a20,a31,a21);     
-        
+            expansion_det2x2(a30,a20,a31,a21);
+
         // Now compute the minors of rank 3
         const expansion& m012_1 = expansion_product(m12,a02);
         expansion& m012_2 = expansion_product(m02,a12); m012_2.negate();
@@ -1106,10 +1283,10 @@ namespace GEO {
 
         const expansion& m013_1 = expansion_product(m13,a02);
         expansion& m013_2 = expansion_product(m03,a12); m013_2.negate();
-        
+
         const expansion& m013_3 = expansion_product(m01,a32);
         const expansion& m013 = expansion_sum3(m013_1, m013_2, m013_3);
-        
+
         const expansion& m023_1 = expansion_product(m23,a02);
         expansion& m023_2 = expansion_product(m03,a22); m023_2.negate();
         const expansion& m023_3 = expansion_product(m02,a32);
@@ -1119,7 +1296,7 @@ namespace GEO {
         expansion& m123_2 = expansion_product(m13,a22); m123_2.negate();
         const expansion& m123_3 = expansion_product(m12,a32);
         const expansion& m123 = expansion_sum3(m123_1, m123_2, m123_3);
-        
+
         // Now compute the minors of rank 4
         const expansion& m0123_1 = expansion_product(m123,a03);
         const expansion& m0123_2 = expansion_product(m023,a13);
@@ -1132,14 +1309,13 @@ namespace GEO {
         const expansion& result = expansion_diff(z1,z2);
         return result.sign();
     }
-    
+
     /************************************************************************/
 
     void expansion::optimize() {
-        grow_expansion_zeroelim(*this, 0.0, *this);
+        compress_expansion(*this);
     }
 
     /************************************************************************/
-    
-}
 
+}
