@@ -13,7 +13,7 @@
  *  * Neither the name of the ALICE Project-Team nor the names of its
  *  contributors may be used to endorse or promote products derived from this
  *  software without specific prior written permission.
- * 
+ *
  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -55,7 +55,7 @@
  *
  *   Il faudra peut-etre quand meme encoder les valeurs de picking de l'etat
  *  sous forme de vec4 plutot qu'en int, parceque je ne suis pas sur que ces
- *  ints puissent stocker plus que 65535 (mais le fait que je voie du bleu 
+ *  ints puissent stocker plus que 65535 (mais le fait que je voie du bleu
  *  quand on met plein de primitive tend a montrer que ca marche quand meme...)
  *
  *   Il manque encore le picking en mode "slice cell"
@@ -72,7 +72,7 @@
  *    faire le deuxi\`eme:
  *       Ca a l'air de marcher pas trop mal.
  *       Rem: le ELEMENT_ARRAY_BUFFER est en entiers 32 bits, certaines archis
- *    (telephones etc...) peuvent avoir besoin d'un ELEMENT_ARRAY_BUFFER 
+ *    (telephones etc...) peuvent avoir besoin d'un ELEMENT_ARRAY_BUFFER
  *    16 bits (ce qu'on pourrait assez facilement avoir...)
  *
  * texture 3D:
@@ -90,7 +90,7 @@
  *     - Linux Intel i915
  *     - Linux NVidia
  *     - Windows
- *  GLSL 1.5 mode is used by destop OpenGL on MacOSX (MacOSX supports 1.5 
+ *  GLSL 1.5 mode is used by destop OpenGL on MacOSX (MacOSX supports 1.5
  *   but does not support 1.3)
  *
  * TODO: In the shaders, in/out declarations are selected by #ifdef GL_ES,
@@ -103,7 +103,7 @@ namespace {
     /**
      * \brief Creates the source of a GLSL function that
      *   gets mesh texture coordinates.
-     * \details This function is necessary because we do not have 
+     * \details This function is necessary because we do not have
      *   a way of specifying uniform constant arrays in OpenGL ES 2.0
      * \param[in] tex_coords an array of 4*\p nb_tex_coords floats
      * \param[in] nb_tex_coords number of vec4s that define texture coordinates
@@ -141,16 +141,36 @@ namespace {
      *  the shader.
      */
 
-#define mesh_tex_coord_lookup(tex)                    \
-    create_mesh_tex_coord_lookup_function(            \
-        tex, index_t(sizeof(tex) / (4*sizeof(float))) \
+#define mesh_tex_coord_lookup(tex)                      \
+    create_mesh_tex_coord_lookup_function(              \
+        tex, index_t(sizeof(tex) / (4*sizeof(float)))   \
     )
 }
 
 namespace GLUP {
 
     extern bool vertex_array_emulate;
-    
+
+    void Context_ES2::begin(GLUPprimitive primitive) {
+        if(primitive == GLUP_THICK_LINES) {
+            // Thick lines use the normal vertex attribute to store the second
+            // extremity of each segment.
+            immediate_state_.buffer[GLUP_NORMAL_ATTRIBUTE].enable();
+        }
+        Context::begin(primitive);
+        if(primitive == GLUP_THICK_LINES) {
+            // Thick lines need to replace line segments with quads.
+            // The additional vertices are generated in
+            // flush()_immediate_buffers (but we need twice the
+            // room in the buffer, so we flush when the buffer is half-full).
+            immediate_state_.begin(GLUP_THICK_LINES, IMMEDIATE_BUFFER_SIZE/2);
+        }
+    }
+
+    void Context_ES2::end() {
+        Context::end();
+    }
+
     const char* Context_ES2::profile_name() const {
         return "GLUPES2";
     }
@@ -170,21 +190,23 @@ namespace GLUP {
     void Context_ES2::done_draw(GLUPprimitive primitive) {
         Context::done_draw(primitive);
     }
-    
+
     bool Context_ES2::primitive_supports_array_mode(GLUPprimitive prim) const {
-        // Note: points, spheres, lines and triangles without mesh can
-	// support array mode, but this will not work with picking,
-	// therefore it is disabled.
+        // Note: points, spheres, lines with width 1
+        // and triangles without mesh can support array mode,
+        // but this will not work with picking, therefore it is disabled.
+        // In Android it is enabled. Additional test (triangles without mesh
+        // and line width 1) are done in MeshGfx.
 #ifdef GEO_OS_ANDROID
-	return
-	    (prim == GLUP_POINTS ||
-	     prim == GLUP_SPHERES ||
-	     prim == GLUP_LINES ||
-	     prim == GLUP_TRIANGLES);
-#else	
+        return
+            (prim == GLUP_POINTS ||
+             prim == GLUP_SPHERES ||
+             prim == GLUP_LINES ||
+             prim == GLUP_TRIANGLES);
+#else
         geo_argused(prim);
-	return false;
-#endif	
+        return false;
+#endif
     }
 
     /**
@@ -201,23 +223,23 @@ namespace GLUP {
         index_t nb_vertices_per_glup_primitive,
         index_t nb_elements_per_glup_primitive
     ) {
-        index_t nb_glup_primitives = 
+        index_t nb_glup_primitives =
             IMMEDIATE_BUFFER_SIZE / nb_vertices_per_glup_primitive;
         return (
             nb_glup_primitives * nb_elements_per_glup_primitive
         );
     }
 
-    
+
     void Context_ES2::setup() {
 
         bool extension_standard_derivatives = false;
         bool extension_vertex_array_object = false;
-        
+
 #ifdef GEO_OS_EMSCRIPTEN
 
 /**
- * \brief Tests whether a WebGL extension is supported.       
+ * \brief Tests whether a WebGL extension is supported.
  * \details WebGL requires getExtension() to be called for initializing
  *  the extension (if supported, this returns an extension object,
  *  else this returns null).
@@ -225,12 +247,12 @@ namespace GLUP {
  * \retval true if the extension is supported
  * \retval false otherwise
  */
-#define WEBGL_EXTENSION_SUPPORTED(ext)                      \
-        EM_ASM_INT({return (                                \
-             Module.ctx.getExtension(ext) == null ? 0 : 1   \
-        );},0) != 0 
+#define WEBGL_EXTENSION_SUPPORTED(ext)                                  \
+        EM_ASM_INT({return (                                            \
+                    Module.ctx.getExtension(ext) == null ? 0 : 1        \
+                );},0) != 0
+
 
-        
         extension_standard_derivatives = WEBGL_EXTENSION_SUPPORTED(
             'OES_standard_derivatives'
         );
@@ -243,17 +265,17 @@ namespace GLUP {
         //TODO: it seems that standard derivatives and vertex array
         //object extension names are not prefixed with GL_OES_ in
         //OpenGL ES mode (but they are in WebGL, oh my....)
-        
+
         if(use_ES_profile_) {
             extension_standard_derivatives =
                 extension_is_supported("GL_OES_standard_derivatives");
 
-	    // ES profile can be artificially set to true for non-NVidia
-	    // GPUs, see constructor.
+            // ES profile can be artificially set to true for non-NVidia
+            // GPUs, see constructor.
             extension_vertex_array_object =
                 extension_is_supported("GL_OES_vertex_array_object") ||
-	        extension_is_supported("GL_ARB_vertex_array_object") ;
-	    
+                extension_is_supported("GL_ARB_vertex_array_object") ;
+
         } else {
             extension_standard_derivatives = true;
             extension_vertex_array_object = true;
@@ -266,11 +288,11 @@ namespace GLUP {
             GLUP::vertex_array_emulate = true;
         }
 
-        Logger::out("GLUP") 
+        Logger::out("GLUP")
             << "OES_standard_derivatives: "
             << extension_standard_derivatives
             << std::endl;
-	
+
         Logger::out("GLUP")
             << "vertex_array_object: "
             << extension_vertex_array_object
@@ -280,7 +302,7 @@ namespace GLUP {
 
 
         /************** VAOs and VBOs for whole cells clipping ******/
-        
+
         //   Compute the maximum number of elements required to draw
         // the vertices in a buffer, for all types of volumetric
         // primitives.
@@ -288,15 +310,15 @@ namespace GLUP {
         max_nb_elements = std::max(max_nb_elements, nb_elements(4,12));
         max_nb_elements = std::max(max_nb_elements, nb_elements(8,36));
         max_nb_elements = std::max(max_nb_elements, nb_elements(6,24));
-        max_nb_elements = std::max(max_nb_elements, nb_elements(5,18));        
-        max_nb_elements = std::max(max_nb_elements, nb_elements(4,12));        
-        
-        nb_clip_cells_elements_ = max_nb_elements;        
+        max_nb_elements = std::max(max_nb_elements, nb_elements(5,18));
+        max_nb_elements = std::max(max_nb_elements, nb_elements(4,12));
+
+        nb_clip_cells_elements_ = max_nb_elements;
         clip_cells_elements_ = new Numeric::uint16[nb_clip_cells_elements_];
 
         glupGenVertexArrays(1,&clip_cells_VAO_);
         glupBindVertexArray(clip_cells_VAO_);
-        
+
         bind_immediate_state_buffers_to_VAO();
 
         glGenBuffers(1, &clip_cells_elements_VBO_);
@@ -314,12 +336,12 @@ namespace GLUP {
         // Shaders in GLSL 2.0 do not support gl_VertexID, we need to
         // emulate it.
         create_vertex_id_VBO();
-        
+
         /************** VAOs and VBOs for sliced cells clipping ******/
 
         max_nb_elements = 6;
         index_t max_nb_vertices = 12;
-        
+
         glupGenVertexArrays(1,&sliced_cells_VAO_);
         glupBindVertexArray(sliced_cells_VAO_);
 
@@ -332,7 +354,7 @@ namespace GLUP {
             max_nb_elements * sizeof(Numeric::uint16),
             nullptr // no need to copy the buffer, it will be overwritten after.
         );
-        
+
         for(index_t i=0; i<4; ++i) {
             glGenBuffers(1, &sliced_cells_vertex_attrib_VBO_[i]);
             glBindBuffer(GL_ARRAY_BUFFER, sliced_cells_vertex_attrib_VBO_[i]);
@@ -359,7 +381,7 @@ namespace GLUP {
 
     Context_ES2::Context_ES2() :
         nb_clip_cells_elements_(0),
-        clip_cells_elements_(nullptr),        
+        clip_cells_elements_(nullptr),
         clip_cells_elements_VBO_(0),
         clip_cells_VAO_(0),
         sliced_cells_elements_VBO_(0),
@@ -368,14 +390,18 @@ namespace GLUP {
         for(index_t i=0; i<4; ++i) {
             sliced_cells_vertex_attrib_VBO_[i] = 0;
         }
-	GLSL_version_ = GLSL::supported_language_version();
+        GLSL_version_ = GLSL::supported_language_version();
 #if defined(GEO_OS_EMSCRIPTEN) || defined(GEO_OS_ANDROID)
         use_ES_profile_ = true;
-	Logger::out("GLUP") << "ES2 context, supported GLSL version = " << GLSL_version_
-			    << std::endl;
-#endif	
+        Logger::out("GLUP") << "ES2 context, supported GLSL version = " << GLSL_version_
+                            << std::endl;
+#endif
+        // GLUP_THICK_LINES are rendered as quads, with four vertices. In this
+        // profile, we have no geometry shader and we need to do the job on our
+        // own in flush_immediate_buffers().
+        nb_vertices_per_primitive_[GLUP_THICK_LINES] = 4;
     }
-    
+
     Context_ES2::~Context_ES2() {
         glDeleteBuffers(1,&clip_cells_elements_VBO_);
         glupDeleteVertexArrays(1, &clip_cells_VAO_);
@@ -385,7 +411,7 @@ namespace GLUP {
         glupDeleteVertexArrays(1, &sliced_cells_VAO_);
         delete[] clip_cells_elements_;
     }
-    
+
     Memory::pointer Context_ES2::get_state_variable_address(
         const char* name
     ) {
@@ -397,25 +423,25 @@ namespace GLUP {
 
     void Context_ES2::do_update_uniform_buffer() {
         GEO_CHECK_GL();
-        
+
         if(!uniform_buffer_dirty_) {
             return;
         }
 
         GEO_CHECK_GL();
-        
+
         if(matrices_dirty_) {
             update_matrices();
         }
 
         GEO_CHECK_GL();
-        
+
         if(lighting_dirty_) {
             update_lighting();
         }
 
         GEO_CHECK_GL();
-        
+
         glUseProgram(latest_program_);
 
         GEO_CHECK_GL();
@@ -423,9 +449,9 @@ namespace GLUP {
         if(latest_program_ != 0) {
             copy_uniform_state_to_current_program();
         }
-        
+
         GEO_CHECK_GL();
-        
+
         uniform_buffer_dirty_ = false;
     }
 
@@ -474,48 +500,48 @@ namespace GLUP {
                         uniform_state_.normal_matrix.get_pointer()[i*4+j];
                 }
             }
-            
+
             glUniformMatrix3fv(loc, 1, GL_FALSE, normal_matrix);
 
 
-	    loc = glGetUniformLocation(
-		latest_program_, "GLUP_VS.inverse_modelview_matrix"		
-	    );
-	    glUniformMatrix4fv(
-		loc, 1, GL_FALSE,
-		uniform_state_.inverse_modelview_matrix.get_pointer()
-	    );
-	    loc = glGetUniformLocation(
-		latest_program_, "GLUP_VS.inverse_projection_matrix"		
-	    );
-	    glUniformMatrix4fv(
-		loc, 1, GL_FALSE,
-		uniform_state_.inverse_projection_matrix.get_pointer()
-	    );
-	    loc = glGetUniformLocation(
-		latest_program_, "GLUP_VS.inverse_modelviewprojection_matrix"
-	    );
-	    glUniformMatrix4fv(
-		loc, 1, GL_FALSE,
-		uniform_state_.inverse_modelviewprojection_matrix.get_pointer()
-	    );
-	    
+            loc = glGetUniformLocation(
+                latest_program_, "GLUP_VS.inverse_modelview_matrix"
+            );
+            glUniformMatrix4fv(
+                loc, 1, GL_FALSE,
+                uniform_state_.inverse_modelview_matrix.get_pointer()
+            );
+            loc = glGetUniformLocation(
+                latest_program_, "GLUP_VS.inverse_projection_matrix"
+            );
+            glUniformMatrix4fv(
+                loc, 1, GL_FALSE,
+                uniform_state_.inverse_projection_matrix.get_pointer()
+            );
+            loc = glGetUniformLocation(
+                latest_program_, "GLUP_VS.inverse_modelviewprojection_matrix"
+            );
+            glUniformMatrix4fv(
+                loc, 1, GL_FALSE,
+                uniform_state_.inverse_modelviewprojection_matrix.get_pointer()
+            );
+
             loc = glGetUniformLocation(
                 latest_program_, "GLUP_VS.viewport"
             );
-	    GLint viewport[4];
-	    glGetIntegerv(GL_VIEWPORT, viewport);
-	    glUniform4f(
-		loc,
-		float(viewport[0]),
-		float(viewport[1]),
-		float(viewport[2]),
-		float(viewport[3])		
-	    );
+            GLint viewport[4];
+            glGetIntegerv(GL_VIEWPORT, viewport);
+            glUniform4f(
+                loc,
+                float(viewport[0]),
+                float(viewport[1]),
+                float(viewport[2]),
+                float(viewport[3])
+            );
         }
 
-	// Matrices (in fragment shader)
-	{
+        // Matrices (in fragment shader)
+        {
             loc = glGetUniformLocation(
                 latest_program_, "GLUP.texture_matrix"
             );
@@ -523,20 +549,20 @@ namespace GLUP {
                 loc, 1, GL_FALSE,
                 uniform_state_.texture_matrix.get_pointer()
             );
-	    loc = glGetUniformLocation(
-		latest_program_, "GLUP.modelviewprojection_matrix"		
-	    );
-	    glUniformMatrix4fv(
-		loc, 1, GL_FALSE,
-		uniform_state_.modelviewprojection_matrix.get_pointer()
-	    );
-	    loc = glGetUniformLocation(
-		latest_program_, "GLUP.inverse_modelviewprojection_matrix"
-	    );
-	    glUniformMatrix4fv(
-		loc, 1, GL_FALSE,
-		uniform_state_.inverse_modelviewprojection_matrix.get_pointer()
-	    );
+            loc = glGetUniformLocation(
+                latest_program_, "GLUP.modelviewprojection_matrix"
+            );
+            glUniformMatrix4fv(
+                loc, 1, GL_FALSE,
+                uniform_state_.modelviewprojection_matrix.get_pointer()
+            );
+            loc = glGetUniformLocation(
+                latest_program_, "GLUP.inverse_modelviewprojection_matrix"
+            );
+            glUniformMatrix4fv(
+                loc, 1, GL_FALSE,
+                uniform_state_.inverse_modelviewprojection_matrix.get_pointer()
+            );
 
             loc = glGetUniformLocation(
                 latest_program_, "GLUP.normal_matrix"
@@ -554,17 +580,17 @@ namespace GLUP {
             loc = glGetUniformLocation(
                 latest_program_, "GLUP.viewport"
             );
-	    GLint viewport[4];
-	    glGetIntegerv(GL_VIEWPORT, viewport);
-	    glUniform4f(
-		loc,
-		float(viewport[0]),
-		float(viewport[1]),
-		float(viewport[2]),
-		float(viewport[3])		
-	    );
-	}
-	
+            GLint viewport[4];
+            glGetIntegerv(GL_VIEWPORT, viewport);
+            glUniform4f(
+                loc,
+                float(viewport[0]),
+                float(viewport[1]),
+                float(viewport[2]),
+                float(viewport[3])
+            );
+        }
+
         // Points (in vertex shader).
         {
             loc = glGetUniformLocation(
@@ -572,8 +598,8 @@ namespace GLUP {
             );
             glUniform1f(loc, uniform_state_.point_size.get());
         }
-        
-        
+
+
         // Lighting.
         {
             loc = glGetUniformLocation(
@@ -607,7 +633,7 @@ namespace GLUP {
                 loc = glGetUniformLocation(
                     latest_program_,
                     ("GLUP." + uniform_state_.color[i].name()).c_str()
-                    );
+                );
                 glUniform4fv(
                     loc, 1, uniform_state_.color[i].get_pointer()
                 );
@@ -620,6 +646,9 @@ namespace GLUP {
             glUniform1f(loc, uniform_state_.mesh_width.get());
         }
 
+        loc = glGetUniformLocation(latest_program_, "GLUP_VS.mesh_width");
+        glUniform1f(loc, uniform_state_.mesh_width.get());
+
         // Texturing.
         if(uniform_state_.toggle[GLUP_TEXTURING].get()) {
             loc = glGetUniformLocation(
@@ -682,29 +711,29 @@ namespace GLUP {
                 latest_program_, "GLUP.alpha_threshold"
             );
             glUniform1f(loc, uniform_state_.alpha_threshold.get());
-	}
+        }
 
-	// specular
-	{
+        // specular
+        {
             loc = glGetUniformLocation(
                 latest_program_, "GLUP.specular"
             );
             glUniform1f(loc, uniform_state_.specular.get());
-	}
+        }
     }
 
     void Context_ES2::update_base_picking_id(GLint new_value) {
         if(uniform_state_.toggle[GLUP_PICKING].get()) {
             uniform_state_.base_picking_id.set(new_value);
             if(latest_program_ != 0) {
-                GEO_CHECK_GL();                                        
+                GEO_CHECK_GL();
                 GLint loc = glGetUniformLocation(
                     latest_program_, "GLUP.base_picking_id"
                 );
                 GEO_CHECK_GL();
                 glUseProgram(latest_program_);
                 glUniform1i(loc, new_value);
-                GEO_CHECK_GL();                
+                GEO_CHECK_GL();
             }
         }
     }
@@ -720,7 +749,7 @@ namespace GLUP {
                 "//stage GL_VERTEX_SHADER\n"
                 "//import <GLUPES/points_vertex_shader.h>\n",
                 "//stage GL_FRAGMENT_SHADER\n"
-                "//import <GLUPES/points_fragment_shader.h>\n"                
+                "//import <GLUPES/points_fragment_shader.h>\n"
             )
         );
     }
@@ -733,11 +762,54 @@ namespace GLUP {
                 "//stage GL_VERTEX_SHADER\n"
                 "//import <GLUPES/vertex_shader.h>\n",
                 "//stage GL_FRAGMENT_SHADER\n"
-                "//import <GLUPES/lines_fragment_shader.h>\n"                
+                "//import <GLUPES/lines_fragment_shader.h>\n"
             )
         );
     }
 
+    void Context_ES2::setup_GLUP_THICK_LINES() {
+        // GLUP_THICK_LINES are rendered as quads,
+        // Only immediate mode is supported.
+        // the line segments given by the user are transformed into quads
+        // in flush_immediate_buffers().
+
+        // For each quad, we draw two triangles
+        static index_t element_indices[6]  = {
+            0, 1, 2,
+            2, 1, 3
+        };
+
+        set_primitive_info_immediate_index_mode(
+            primitive_source_, GL_TRIANGLES,
+            GLSL::compile_program_with_includes_no_link(
+                this,
+                "//stage GL_VERTEX_SHADER\n"
+                "//import <GLUPES/thick_lines_vertex_shader.h>\n",
+                "//stage GL_FRAGMENT_SHADER\n"
+                "//import <GLUPES/thick_lines_fragment_shader.h>\n"
+            ),
+            index_t(sizeof(element_indices)/sizeof(index_t)),
+            element_indices
+        );
+
+        // The number of vertices per thick line segment is adapted in two
+        // other places in the code:
+        // - in Context_ES2::begin(), the maximum index for the immediate
+        //   buffers is set to half-buffer (so that we have enough room to
+        //   generate the additional vertices to render quads).
+        // - in Context_ES2::flush_immediate_buffers(), the immediate buffers
+        //   are reshuffled, in order to create quads, with
+        //   their four vertices set to the first vertex of each segment,
+        //   and "normals" set the second vertex of each segment.
+        //
+        // The vertex shader uses the primitive Id to know which vertex
+        // goes where.
+        //
+        // Note: since the GLUP_NORMAL_ATTRIBUTE is used to store the second
+        // vertex of each segment, it needs to be bound. It is done in
+        // Context_ES2::begin() (that calls then Context::begin()).
+    }
+
     void Context_ES2::setup_primitive_generic(
         index_t nb_elements_per_primitive,
         index_t* element_indices
@@ -750,12 +822,12 @@ namespace GLUP {
                 "//stage GL_VERTEX_SHADER\n"
                 "//import <GLUPES/vertex_shader.h>\n",
                 "//stage GL_FRAGMENT_SHADER\n"
-                "//import <GLUPES/fragment_shader.h>\n"                
+                "//import <GLUPES/fragment_shader.h>\n"
             ),
             nb_elements_per_primitive, element_indices
         );
     }
-    
+
     void Context_ES2::setup_GLUP_TRIANGLES() {
         static index_t element_indices[3]  = {
             0, 1, 2
@@ -853,11 +925,11 @@ namespace GLUP {
                 "//stage GL_VERTEX_SHADER\n"
                 "//import <GLUPES/spheres_vertex_shader.h>\n",
                 "//stage GL_FRAGMENT_SHADER\n"
-                "//import <GLUPES/spheres_fragment_shader.h>\n"                
+                "//import <GLUPES/spheres_fragment_shader.h>\n"
             )
         );
     }
-    
+
     bool Context_ES2::cell_by_cell_clipping() const {
         if(!uniform_state_.toggle[GLUP_CLIPPING].get()) {
             return false;
@@ -868,11 +940,11 @@ namespace GLUP {
             immediate_state_.primitive() != GLUP_HEXAHEDRA &&
             immediate_state_.primitive() != GLUP_PRISMS &&
             immediate_state_.primitive() != GLUP_PYRAMIDS &&
-            immediate_state_.primitive() != GLUP_CONNECTORS 
+            immediate_state_.primitive() != GLUP_CONNECTORS
         ) {
             return false;
         }
-        
+
         return (
             uniform_state_.clipping_mode.get() == GLUP_CLIP_WHOLE_CELLS ||
             uniform_state_.clipping_mode.get() == GLUP_CLIP_STRADDLING_CELLS
@@ -889,21 +961,21 @@ namespace GLUP {
             immediate_state_.primitive() != GLUP_HEXAHEDRA &&
             immediate_state_.primitive() != GLUP_PRISMS &&
             immediate_state_.primitive() != GLUP_PYRAMIDS &&
-            immediate_state_.primitive() != GLUP_CONNECTORS 
+            immediate_state_.primitive() != GLUP_CONNECTORS
         ) {
             return false;
         }
-        
+
         return (
-            uniform_state_.clipping_mode.get() == GLUP_CLIP_SLICE_CELLS 
+            uniform_state_.clipping_mode.get() == GLUP_CLIP_SLICE_CELLS
         );
     }
-    
+
     void Context_ES2::flush_immediate_buffers_with_cell_by_cell_clipping() {
         index_t cur_vertex = 0;
         index_t cur_element_out = 0;
         index_t nb_vertices_per_cell =
-            nb_vertices_per_primitive[immediate_state_.primitive()];
+            nb_vertices_per_primitive_[immediate_state_.primitive()];
         index_t nb_elements_per_cell = primitive_info_[
             immediate_state_.primitive()].nb_elements_per_primitive;
 
@@ -947,32 +1019,32 @@ namespace GLUP {
 
         if(
             vertex_id_VBO_ != 0 && ((
-                immediate_state_.primitive() >= GLUP_TRIANGLES &&
-                uniform_state_.toggle[GLUP_DRAW_MESH].get()
-            ) || uniform_state_.toggle[GLUP_PICKING].get())
+                                        immediate_state_.primitive() >= GLUP_TRIANGLES &&
+                                        uniform_state_.toggle[GLUP_DRAW_MESH].get()
+                                    ) || uniform_state_.toggle[GLUP_PICKING].get())
         ) {
-	    if(!vertex_id_VBO_bound_) {
-		glEnableVertexAttribArray(GLUP_VERTEX_ID_ATTRIBUTE);
-		glBindBuffer(
-		    GL_ARRAY_BUFFER, vertex_id_VBO_
-		);
-		glVertexAttribPointer(
-		    GLUP_VERTEX_ID_ATTRIBUTE,
-		    1,                 // 1 component per attribute
-		    GL_UNSIGNED_SHORT, // components are 16 bits integers
-		    GL_FALSE,          // do not normalize
-		    0,                 // stride
-		    nullptr            // pointer (relative to bound VBO beginning)
-		);
-		vertex_id_VBO_bound_ = true;
-	    }
+            if(!vertex_id_VBO_bound_) {
+                glEnableVertexAttribArray(GLUP_VERTEX_ID_ATTRIBUTE);
+                glBindBuffer(
+                    GL_ARRAY_BUFFER, vertex_id_VBO_
+                );
+                glVertexAttribPointer(
+                    GLUP_VERTEX_ID_ATTRIBUTE,
+                    1,              // 1 component per attribute
+                    GL_UNSIGNED_SHORT, // components are 16 bits integers
+                    GL_FALSE,       // do not normalize
+                    0,              // stride
+                    nullptr         // pointer (relative to bound VBO beginning)
+                );
+                vertex_id_VBO_bound_ = true;
+            }
         } else {
-	    if(vertex_id_VBO_bound_) {
-		vertex_id_VBO_bound_ = false;
-		glDisableVertexAttribArray(GLUP_VERTEX_ID_ATTRIBUTE);
-	    }
+            if(vertex_id_VBO_bound_) {
+                vertex_id_VBO_bound_ = false;
+                glDisableVertexAttribArray(GLUP_VERTEX_ID_ATTRIBUTE);
+            }
         }
-        
+
         // Stream the indices into the elements VBO.
         stream_buffer_object(
             clip_cells_elements_VBO_,
@@ -980,21 +1052,21 @@ namespace GLUP {
             cur_element_out * sizeof(Numeric::uint16),
             clip_cells_elements_
         );
-        
+
         glDrawElements(
             primitive_info_[immediate_state_.primitive()].GL_primitive,
             GLsizei(cur_element_out),
             GL_UNSIGNED_SHORT,
             nullptr
         );
-        
+
         glupBindVertexArray(0);
         glDisableVertexAttribArray(GLUP_VERTEX_ID_ATTRIBUTE);
         immediate_state_.reset();
     }
 
     void Context_ES2::flush_immediate_buffers_with_sliced_cells_clipping() {
-        
+
         MarchingCell* marching_cell = nullptr;
         switch(immediate_state_.primitive()) {
         case GLUP_TETRAHEDRA:
@@ -1014,9 +1086,10 @@ namespace GLUP {
             break;
         case GLUP_POINTS:
         case GLUP_LINES:
+        case GLUP_THICK_LINES:
         case GLUP_TRIANGLES:
         case GLUP_QUADS:
-	case GLUP_SPHERES:
+        case GLUP_SPHERES:
         case GLUP_NB_PRIMITIVES:
             geo_assert_not_reached;
         }
@@ -1039,7 +1112,7 @@ namespace GLUP {
             }
 
             if(marching_cell->config_size(config) != 0) {
-                
+
                 stream_buffer_object(
                     sliced_cells_elements_VBO_,
                     GL_ELEMENT_ARRAY_BUFFER,
@@ -1070,28 +1143,96 @@ namespace GLUP {
 
                 glDrawElements(
                     GL_TRIANGLE_FAN,
-                    GLsizei(marching_cell->config_size(config)), 
+                    GLsizei(marching_cell->config_size(config)),
                     GL_UNSIGNED_INT,
                     nullptr
                 );
             }
-            
+
             v0 += marching_cell->nb_vertices();
         }
-        
-        glupBindVertexArray(0);        
+
+        glupBindVertexArray(0);
         immediate_state_.reset();
     }
-    
+
     void Context_ES2::flush_immediate_buffers() {
-        classify_vertices_in_immediate_buffers();        
+        if(immediate_state_.primitive() == GLUP_THICK_LINES) {
+            // Make sure we have enough room to double the number of
+            // vertices in immediate buffers
+            geo_assert(
+                immediate_state_.nb_vertices() <= IMMEDIATE_BUFFER_SIZE/2
+            );
+
+            // input:
+            // (v1 v2), (v1 v2), (v1 v2) in vertex buffer
+            //
+            // output:
+            // (v1 v1 v1 v1) (v1 v1 v1 v1) in vertex buffer
+            // (v2 v2 v2 v2) (v2 v2 v2 v2) in normal buffer
+            //
+            // Each line segment becomes four identical vertices. Vertex
+            // shader distinguished them based on (vertex_id % 4).
+
+            for(index_t v=0; v<immediate_state_.nb_vertices(); ++v) {
+                index_t from = immediate_state_.nb_vertices()-v-1;
+                index_t to   = 2*from;
+                immediate_state_.copy_element(to,from);
+            }
+
+            for(index_t v=0; v<immediate_state_.nb_vertices(); ++v) {
+                index_t from = 2*v;
+                index_t to   = 2*v+1;
+                immediate_state_.copy_element(to,from);
+            }
+
+            // This doubles immediate_state_.nb_vertices()
+            // (and does nothing else)
+            immediate_state_.reset(immediate_state_.nb_vertices()*2);
+
+            for(index_t v1=0; v1<immediate_state_.nb_vertices(); v1+=4) {
+                index_t v2=v1+1;
+                index_t v3=v1+2;
+                index_t v4=v1+3;
+
+                // copy second extremity to normal attribute
+                // of the four vertices
+                immediate_state_.buffer[GLUP_NORMAL_ATTRIBUTE].copy(
+                    v1,
+                    immediate_state_.buffer[GLUP_VERTEX_ATTRIBUTE],
+                    v3
+                );
+                immediate_state_.buffer[GLUP_NORMAL_ATTRIBUTE].copy(
+                    v2,
+                    immediate_state_.buffer[GLUP_VERTEX_ATTRIBUTE],
+                    v3
+                );
+                immediate_state_.buffer[GLUP_NORMAL_ATTRIBUTE].copy(
+                    v3,
+                    immediate_state_.buffer[GLUP_VERTEX_ATTRIBUTE],
+                    v3
+                );
+                immediate_state_.buffer[GLUP_NORMAL_ATTRIBUTE].copy(
+                    v4,
+                    immediate_state_.buffer[GLUP_VERTEX_ATTRIBUTE],
+                    v3
+                );
+
+                // copy first extremity to vertex attribute of the four vertices
+                // (three in fact, first one is already there)
+                immediate_state_.buffer[GLUP_VERTEX_ATTRIBUTE].copy(v2,v1);
+                immediate_state_.buffer[GLUP_VERTEX_ATTRIBUTE].copy(v3,v1);
+                immediate_state_.buffer[GLUP_VERTEX_ATTRIBUTE].copy(v4,v1);
+            }
+        }
+        classify_vertices_in_immediate_buffers();
         shrink_cells_in_immediate_buffers();
         if(cell_by_cell_clipping()) {
-            flush_immediate_buffers_with_cell_by_cell_clipping();            
+            flush_immediate_buffers_with_cell_by_cell_clipping();
         } else if(sliced_cells_clipping()) {
             flush_immediate_buffers_with_sliced_cells_clipping();
         } else {
-            Context::flush_immediate_buffers();            
+            Context::flush_immediate_buffers();
         }
     }
 
@@ -1100,48 +1241,48 @@ namespace GLUP {
         std::vector<GLSL::Source>& sources
     ) {
         Context::get_primitive_pseudo_file(sources);
-        
+
         static GLUPfloat tri_tex_coords[12] = {
             1.0f, 0.0f, 0.0f, 0.0f,
             0.0f, 1.0f, 0.0f, 0.0f,
-            0.0f, 0.0f, 1.0f, 0.0f            
+            0.0f, 0.0f, 1.0f, 0.0f
         };
 
         static GLUPfloat quad_tex_coords[16] = {
             0.0f, 0.0f, 1.0f, 1.0f,
             1.0f, 0.0f, 0.0f, 1.0f,
             1.0f, 1.0f, 0.0f, 0.0f,
-            0.0f, 1.0f, 1.0f, 0.0f,            
+            0.0f, 1.0f, 1.0f, 0.0f,
         };
-        
+
         static GLUPfloat tet_tex_coords[16] = {
             1.0f, 0.0f, 0.0f, 0.0f,
             0.0f, 1.0f, 0.0f, 0.0f,
             0.0f, 0.0f, 1.0f, 0.0f,
-            0.0f, 0.0f, 0.0f, 1.0f            
+            0.0f, 0.0f, 0.0f, 1.0f
         };
 
         static GLUPfloat hex_tex_coords[32] = {
-            0.0f, 0.0f, 0.0f, 0.0f,            
+            0.0f, 0.0f, 0.0f, 0.0f,
             0.0f, 0.0f, 1.0f, 0.0f,
             0.0f, 1.0f, 0.0f, 0.0f,
             0.0f, 1.0f, 1.0f, 0.0f,
-            1.0f, 0.0f, 0.0f, 0.0f,            
+            1.0f, 0.0f, 0.0f, 0.0f,
             1.0f, 0.0f, 1.0f, 0.0f,
             1.0f, 1.0f, 0.0f, 0.0f,
             1.0f, 1.0f, 1.0f, 0.0f,
         };
 
         static GLUPfloat prism_tex_coords[24] = {
-            1.0f, 0.0f, 0.0f, 0.0f,            
+            1.0f, 0.0f, 0.0f, 0.0f,
             0.0f, 1.0f, 0.0f, 0.0f,
             0.0f, 0.0f, 1.0f, 0.0f,
-            1.0f, 0.0f, 0.0f, 1.0f,            
+            1.0f, 0.0f, 0.0f, 1.0f,
             0.0f, 1.0f, 0.0f, 1.0f,
             0.0f, 0.0f, 1.0f, 1.0f
         };
 
-        
+
         switch(primitive_source_) {
         case GLUP_TRIANGLES:
             sources.push_back(mesh_tex_coord_lookup(tri_tex_coords));
@@ -1153,17 +1294,18 @@ namespace GLUP {
             sources.push_back(mesh_tex_coord_lookup(tet_tex_coords));
             break;
         case GLUP_HEXAHEDRA:
-            sources.push_back(mesh_tex_coord_lookup(hex_tex_coords));    
+            sources.push_back(mesh_tex_coord_lookup(hex_tex_coords));
             break;
         case GLUP_PRISMS:
             sources.push_back(mesh_tex_coord_lookup(prism_tex_coords));
             break;
-            
+
         case GLUP_POINTS:
         case GLUP_LINES:
+        case GLUP_THICK_LINES:
         case GLUP_PYRAMIDS:
         case GLUP_CONNECTORS:
-	case GLUP_SPHERES:
+        case GLUP_SPHERES:
             sources.push_back(
                 "#define GLUP_NO_MESH_TEX_COORDS\n"
                 "vec4 get_mesh_tex_coord(in int vertex_id) {\n"
@@ -1181,49 +1323,49 @@ namespace GLUP {
         std::vector<GLSL::Source>& sources
     ) {
         if(use_ES_profile_) {
-	    if(GLSL_version_ >= 3.0) {
-		sources.push_back(
-		    "#version 300 es\n"
-		    "#define GLUP_VERTEX_SHADER \n"
-		);
-	    } else {
-		sources.push_back(
-		    "#version 100\n"
-		    "#define GLUP_VERTEX_SHADER \n"
-		);
-	    }
+            if(GLSL_version_ >= 3.0) {
+                sources.push_back(
+                    "#version 300 es\n"
+                    "#define GLUP_VERTEX_SHADER \n"
+                );
+            } else {
+                sources.push_back(
+                    "#version 100\n"
+                    "#define GLUP_VERTEX_SHADER \n"
+                );
+            }
         } else {
             sources.push_back(
-#ifdef GEO_OS_APPLE		
-		"#version 150 core          \n"
-#else		
+#ifdef GEO_OS_APPLE
+                "#version 150 core          \n"
+#else
                 "#version 130               \n"
-#endif		
+#endif
                 "#define GLUP_VERTEX_SHADER \n"
             );
         }
     }
-    
+
 
     void Context_ES2::get_fragment_shader_preamble_pseudo_file(
         std::vector<GLSL::Source>& sources
     ) {
         if(use_ES_profile_) {
 #if defined(GEO_OS_EMSCRIPTEN) || defined(GEO_OS_ANDROID)
-	    if(GLSL_version_ >= 3.0) {
-		sources.push_back("#version 300 es\n");
-	    } else {
-		sources.push_back("#version 100\n");		
-	    }
-#endif	    
-	    sources.push_back(
+            if(GLSL_version_ >= 3.0) {
+                sources.push_back("#version 300 es\n");
+            } else {
+                sources.push_back("#version 100\n");
+            }
+#endif
+            sources.push_back(
                 "#define GLUP_FRAGMENT_SHADER                    \n"
                 "#extension GL_OES_standard_derivatives : enable \n"
                 "#extension GL_EXT_frag_depth : enable           \n"
-		"#extension GL_OES_texture3D : enable            \n"		
-		"#ifndef GL_OES_texture3D                        \n"
-		"   #define GLUP_NO_TEXTURE_3D                   \n"		
-		"#endif                                          \n"
+                "#extension GL_OES_texture3D : enable            \n"
+                "#ifndef GL_OES_texture3D                        \n"
+                "   #define GLUP_NO_TEXTURE_3D                   \n"
+                "#endif                                          \n"
                 "precision highp float;                          \n"
                 "#ifdef GL_FRAGMENT_PRECISION_HIGH               \n"
                 "   precision highp int;                         \n"
@@ -1231,20 +1373,20 @@ namespace GLUP {
             );
         } else {
             sources.push_back(
-#ifdef GEO_OS_APPLE		
-		"#version 150 core                               \n"
-#else		
+#ifdef GEO_OS_APPLE
+                "#version 150 core                               \n"
+#else
                 "#version 130                                    \n"
-#endif		
+#endif
                 "#define GLUP_FRAGMENT_SHADER                    \n"
             );
         }
     }
 
     void Context_ES2::get_toggles_pseudo_file(
-        std::vector<GLSL::Source>& sources        
+        std::vector<GLSL::Source>& sources
     ) {
-        
+
         //   In GLUPES2, the state is split into vertex shader
         // and fragment shader state. The toggles are in the fragment
         // shader state, therefore the vertex shader can only know the
@@ -1252,8 +1394,8 @@ namespace GLUP {
         // "conservatively" assume that they are on (on the worst case,
         // this means the vertex shader will copy more attributes than
         // needed).
-        
-        std::string toggle_maybe_enabled_source = 
+
+        std::string toggle_maybe_enabled_source =
             "bool glupIsEnabled(in int toggle) {\n";
 
         for(index_t i=0; i<uniform_state_.toggle.size(); ++i) {
@@ -1263,7 +1405,7 @@ namespace GLUP {
             geo_assert(pos != std::string::npos);
             toggle_name = "GLUP_" +
                 String::to_uppercase(toggle_name.substr(0,pos));
-            
+
             std::string test =
                 "   if(toggle=="+toggle_name + ") {\n";
 
@@ -1274,13 +1416,13 @@ namespace GLUP {
             } else {
                 if(toggles_source_state_ & (1 << i)) {
                     toggle_maybe_enabled_source +=
-                        "      return true;\n";                        
+                        "      return true;\n";
                 } else {
                     toggle_maybe_enabled_source +=
-                        "      return false;\n";                        
+                        "      return false;\n";
                 }
             }
-            toggle_maybe_enabled_source += "   }\n";            
+            toggle_maybe_enabled_source += "   }\n";
         }
 
         toggle_maybe_enabled_source += "   return false; \n}\n";
@@ -1294,4 +1436,3 @@ namespace GLUP {
 }
 
 #endif
-
